import os
from tkinter import *
from typing import Union

from simba.mixins.config_reader import ConfigReader
from simba.mixins.pop_up_mixin import PopUpMixin
from simba.ui.tkinter_functions import (CreateLabelFrameWithIcon, Entry_Box,
                                        FileSelect)
from simba.utils.checks import (check_file_exist_and_readable,
                                check_if_string_value_is_valid_video_timestamp,
                                check_int,
                                check_that_hhmmss_start_is_before_end)
from simba.utils.data import find_frame_numbers_from_time_stamp
from simba.utils.enums import Formats, Keys, Links, Options
from simba.utils.read_write import (
    check_if_hhmmss_timestamp_is_valid_part_of_video, find_video_of_file,
    get_fn_ext, read_df, read_video_info, remove_a_folder, write_df)
from simba.video_processors.video_processing import multi_split_video


class TargetedAnnotationsWClipsPopUp(PopUpMixin, ConfigReader):
    def __init__(self, config_path: Union[str, os.PathLike]):
        PopUpMixin.__init__(self, title="TAGETED ANNOTATIONS WITH CLIPS")
        ConfigReader.__init__(self, config_path=config_path)
        settings_frm = CreateLabelFrameWithIcon(
            parent=self.main_frm,
            header="Split videos into different parts",
            icon_name=Keys.DOCUMENTATION.value,
            icon_link=Links.VIDEO_TOOLS.value,
        )
        self.selected_video = FileSelect(
            settings_frm,
            "Data path",
            title="Select a data file",
            lblwidth=10,
            file_types=[("VIDEO FILE", Options.WORKFLOW_FILE_TYPE_OPTIONS.value)],
        )
        self.clip_cnt = Entry_Box(
            settings_frm, "# of clips", "10", validation="numeric"
        )
        confirm_settings_btn = Button(
            settings_frm, text="Confirm", command=lambda: self.show_start_stop()
        )
        settings_frm.grid(row=0, column=0, sticky=NW)
        self.selected_video.grid(row=0, column=0, sticky=NW)
        self.clip_cnt.grid(row=1, column=0, sticky=NW)
        confirm_settings_btn.grid(row=2, column=0, sticky=NW)
        self.main_frm.mainloop()

    def show_start_stop(self):
        check_int(name="Number of clips", value=self.clip_cnt.entry_get)
        if hasattr(self, "table"):
            self.table.destroy()
        self.table = LabelFrame(self.main_frm)
        self.table.grid(row=2, column=0, sticky=NW)
        Label(self.table, text="Clip #").grid(row=0, column=0)
        Label(self.table, text="Start Time").grid(row=0, column=1, sticky=NW)
        Label(self.table, text="Stop Time").grid(row=0, column=2, sticky=NW)
        self.clip_names, self.start_times, self.end_times = [], [], []
        for i in range(int(self.clip_cnt.entry_get)):
            Label(self.table, text="Clip " + str(i + 1)).grid(row=i + 2, sticky=W)
            self.start_times.append(Entry(self.table))
            self.start_times[i].insert(0, "00:00:00")
            self.start_times[i].grid(row=i + 2, column=1, sticky=NW)
            self.end_times.append(Entry(self.table))
            self.end_times[i].insert(0, "00:00:00")
            self.end_times[i].grid(row=i + 2, column=2, sticky=NW)
        run_button = Button(
            self.table,
            text="RUN",
            command=lambda: self.run(),
            fg="navy",
            font=Formats.LABELFRAME_HEADER_FORMAT.value,
        )
        run_button.grid(row=int(self.clip_cnt.entry_get) + 2, column=2, sticky=W)

    def run(self):
        check_file_exist_and_readable(self.selected_video.file_path)
        _, file_name, _ = get_fn_ext(filepath=self.selected_video.file_path)
        machine_results_path = os.path.join(
            self.machine_results_dir, f"{file_name}.{self.file_type}"
        )
        check_file_exist_and_readable(file_path=machine_results_path)
        _, _, fps = read_video_info(
            vid_info_df=self.video_info_df, video_name=file_name
        )
        video_path = find_video_of_file(
            video_dir=self.video_dir, filename=file_name, raise_error=True
        )
        start_times, end_times = [], []
        check_file_exist_and_readable(self.selected_video.file_path)
        for cnt, (start_time, end_time) in enumerate(
            zip(self.start_times, self.end_times)
        ):
            start_times.append(start_time.get())
            end_times.append(end_time.get())
            check_if_string_value_is_valid_video_timestamp(
                name=f"Clip {cnt+1} start", value=start_time.get()
            )
            check_if_string_value_is_valid_video_timestamp(
                name=f"Clip {cnt+1} end", value=end_time.get()
            )
            check_that_hhmmss_start_is_before_end(
                start_time=start_time.get(),
                end_time=end_time.get(),
                name=f"Clip {cnt+1}",
            )
            check_if_hhmmss_timestamp_is_valid_part_of_video(
                timestamp=start_time.get(), video_path=video_path
            )
            check_if_hhmmss_timestamp_is_valid_part_of_video(
                timestamp=end_time.get(), video_path=video_path
            )

        print("Preparing video and data clips...")
        temp_dir = os.path.join(
            self.input_frames_dir, "advanced_clip_annotator", file_name
        )
        if os.path.isdir(temp_dir):
            remove_a_folder(folder_dir=temp_dir)
        if not os.path.isdir(temp_dir):
            os.makedirs(temp_dir)
        multi_split_video(
            file_path=video_path,
            start_times=start_times,
            end_times=end_times,
            out_dir=temp_dir,
            include_clip_time_in_filename=True,
        )

        df = read_df(file_path=machine_results_path, file_type=self.file_type)
        for cnt, (start_time, end_time) in enumerate(zip(start_times, end_times)):
            frm_numbers = find_frame_numbers_from_time_stamp(
                start_time=start_time, end_time=end_time, fps=fps
            )
            sliced = df.iloc[frm_numbers, :]
            new_start_time = start_time.replace(":", "-")
            new_end_time = start_time.replace(":", "-")
            save_path = os.path.join(
                temp_dir,
                f"{file_name}_{new_start_time}_{new_end_time}.{self.file_type}",
            )
            write_df(df=sliced, file_type=self.file_type, save_path=save_path)
            print(f"Data clip {cnt+1} saved...")
        self.root.destroy()
        self.main_frm.destroy()


# TargetedAnnotationsWClipsPopUp(config_path='/Users/simon/Desktop/envs/troubleshooting/two_black_animals_14bp/project_folder/project_config.ini')



__author__ = "Simon Nilsson"

import os
from tkinter import *

from simba.mixins.config_reader import ConfigReader
from simba.mixins.pop_up_mixin import PopUpMixin
from simba.plotting.probability_plot_creator import \
    TresholdPlotCreatorSingleProcess
from simba.plotting.probability_plot_creator_mp import \
    TresholdPlotCreatorMultiprocess
from simba.ui.tkinter_functions import (CreateLabelFrameWithIcon, DropDownMenu,
                                        Entry_Box)
from simba.utils.checks import check_int
from simba.utils.enums import Formats, Keys, Links, Paths
from simba.utils.read_write import (check_if_filepath_list_is_empty,
                                    get_file_name_info_in_directory)


class VisualizeClassificationProbabilityPopUp(PopUpMixin, ConfigReader):
    def __init__(self, config_path: str):
        PopUpMixin.__init__(self, title="CREATE CLASSIFICATION PROBABILITY PLOTS")
        ConfigReader.__init__(self, config_path=config_path)
        self.max_y_lst = [x for x in range(10, 110, 10)]
        self.max_y_lst.insert(0, "auto")
        self.data_path = os.path.join(
            self.project_path, Paths.MACHINE_RESULTS_DIR.value
        )
        self.files_found_dict = get_file_name_info_in_directory(
            directory=self.data_path, file_type=self.file_type
        )
        check_if_filepath_list_is_empty(
            filepaths=list(self.files_found_dict.keys()),
            error_msg="SIMBA ERROR: Cant visualize probabilities, no data in project_folder/csv/machine_results directory",
        )

        self.style_settings_frm = CreateLabelFrameWithIcon(
            parent=self.main_frm,
            header="STYLE SETTINGS",
            icon_name=Keys.DOCUMENTATION.value,
            icon_link=Links.VISUALIZE_CLF_PROBABILITIES.value,
        )
        self.resolution_dropdown = DropDownMenu(
            self.style_settings_frm, "Resolution:", self.resolutions, "16"
        )
        self.max_y_dropdown = DropDownMenu(
            self.style_settings_frm, "Max Y-axis:", self.max_y_lst, "16"
        )

        self.line_clr_dropdown = DropDownMenu(
            self.style_settings_frm, "Line color:", self.colors, "16"
        )
        self.font_size_entry = Entry_Box(
            self.style_settings_frm, "Font size: ", "16", validation="numeric"
        )
        self.line_width = Entry_Box(
            self.style_settings_frm, "Line width: ", "16", validation="numeric"
        )
        self.circle_size = Entry_Box(
            self.style_settings_frm, "Circle size: ", "16", validation="numeric"
        )
        self.line_clr_dropdown.setChoices("blue")
        self.resolution_dropdown.setChoices(self.resolutions[1])
        self.font_size_entry.entry_set(val=10)
        self.line_width.entry_set(val=6)
        self.circle_size.entry_set(val=20)
        self.max_y_dropdown.setChoices(choice="auto")

        self.settings_frm = LabelFrame(
            self.main_frm,
            text="VISUALIZATION SETTINGS",
            font=Formats.LABELFRAME_HEADER_FORMAT.value,
            pady=5,
            padx=5,
        )
        self.probability_frames_var = BooleanVar()
        self.probability_videos_var = BooleanVar()
        self.probability_last_frm_var = BooleanVar()
        self.probability_multiprocess_var = BooleanVar()

        self.clf_dropdown = DropDownMenu(
            self.settings_frm, "Classifier:", self.clf_names, "16"
        )
        self.clf_dropdown.setChoices(self.clf_names[0])
        probability_frames_cb = Checkbutton(
            self.settings_frm,
            text="Create frames",
            variable=self.probability_frames_var,
        )
        probability_videos_cb = Checkbutton(
            self.settings_frm,
            text="Create videos",
            variable=self.probability_videos_var,
        )
        probability_last_frm_cb = Checkbutton(
            self.settings_frm,
            text="Create last frame",
            variable=self.probability_last_frm_var,
        )
        probability_multiprocess_cb = Checkbutton(
            self.settings_frm,
            text="Multi-process (faster)",
            variable=self.probability_multiprocess_var,
            command=lambda: self.enable_dropdown_from_checkbox(
                check_box_var=self.probability_multiprocess_var,
                dropdown_menus=[self.multiprocess_dropdown],
            ),
        )

        self.multiprocess_dropdown = DropDownMenu(
            self.settings_frm, "CPU cores:", list(range(2, self.cpu_cnt)), "12"
        )
        self.multiprocess_dropdown.setChoices(2)
        self.multiprocess_dropdown.disable()

        self.run_frm = LabelFrame(
            self.main_frm,
            text="RUN",
            font=Formats.LABELFRAME_HEADER_FORMAT.value,
            pady=5,
            padx=5,
            fg="black",
        )
        self.run_single_video_frm = LabelFrame(
            self.run_frm,
            text="SINGLE VIDEO",
            font=Formats.LABELFRAME_HEADER_FORMAT.value,
            pady=5,
            padx=5,
            fg="black",
        )
        self.run_single_video_btn = Button(
            self.run_single_video_frm,
            text="Create single video",
            fg="blue",
            command=lambda: self.__create_probability_plots(multiple_videos=False),
        )
        self.single_video_dropdown = DropDownMenu(
            self.run_single_video_frm,
            "Video:",
            list(self.files_found_dict.keys()),
            "12",
        )
        self.single_video_dropdown.setChoices(list(self.files_found_dict.keys())[0])

        self.run_multiple_videos = LabelFrame(
            self.run_frm,
            text="MULTIPLE VIDEO",
            font=Formats.LABELFRAME_HEADER_FORMAT.value,
            pady=5,
            padx=5,
            fg="black",
        )
        self.run_multiple_video_btn = Button(
            self.run_multiple_videos,
            text="Create multiple videos ({} video(s) found)".format(
                str(len(list(self.files_found_dict.keys())))
            ),
            fg="blue",
            command=lambda: self.__create_probability_plots(multiple_videos=True),
        )

        self.style_settings_frm.grid(row=0, sticky=NW)
        self.resolution_dropdown.grid(row=0, sticky=NW)
        self.line_clr_dropdown.grid(row=1, sticky=NW)
        self.font_size_entry.grid(row=2, sticky=NW)
        self.line_width.grid(row=3, sticky=NW)
        self.circle_size.grid(row=4, sticky=NW)
        self.max_y_dropdown.grid(row=5, sticky=NW)

        self.settings_frm.grid(row=1, sticky=NW)
        self.clf_dropdown.grid(row=0, sticky=NW)
        probability_frames_cb.grid(row=1, sticky=NW)
        probability_videos_cb.grid(row=2, sticky=NW)
        probability_last_frm_cb.grid(row=3, sticky=NW)
        probability_multiprocess_cb.grid(row=4, column=0, sticky=W)
        self.multiprocess_dropdown.grid(row=4, column=1, sticky=W)

        self.run_frm.grid(row=2, sticky=NW)
        self.run_single_video_frm.grid(row=0, sticky=NW)
        self.run_single_video_btn.grid(row=0, sticky=NW)
        self.single_video_dropdown.grid(row=0, column=1, sticky=NW)

        self.run_multiple_videos.grid(row=1, sticky=NW)
        self.run_multiple_video_btn.grid(row=0, sticky=NW)

        # self.main_frm.mainloop()

    def __create_probability_plots(self, multiple_videos: bool):
        width = int(self.resolution_dropdown.getChoices().split("×")[0])
        height = int(self.resolution_dropdown.getChoices().split("×")[1])
        check_int(
            name="PLOT FONT SIZE", value=self.font_size_entry.entry_get, min_value=1
        )
        check_int(name="PLOT LINE WIDTH", value=self.line_width.entry_get, min_value=1)
        check_int(
            name="PLOT CIRCLE SIZE", value=self.circle_size.entry_get, min_value=1
        )
        style_attr = {
            "width": width,
            "height": height,
            "font size": int(self.font_size_entry.entry_get),
            "line width": int(self.line_width.entry_get),
            "color": self.line_clr_dropdown.getChoices(),
            "circle size": int(self.circle_size.entry_get),
            "y_max": self.max_y_dropdown.getChoices(),
        }

        if multiple_videos:
            data_paths = list(self.files_found_dict.values())
        else:
            data_paths = [
                self.files_found_dict[self.single_video_dropdown.getChoices()]
            ]

        if not self.probability_multiprocess_var.get():
            probability_plot_creator = TresholdPlotCreatorSingleProcess(
                config_path=self.config_path,
                frame_setting=self.probability_frames_var.get(),
                video_setting=self.probability_videos_var.get(),
                last_image=self.probability_last_frm_var.get(),
                files_found=data_paths,
                clf_name=self.clf_dropdown.getChoices(),
                style_attr=style_attr,
            )
        else:
            probability_plot_creator = TresholdPlotCreatorMultiprocess(
                config_path=self.config_path,
                frame_setting=self.probability_frames_var.get(),
                video_setting=self.probability_videos_var.get(),
                last_frame=self.probability_last_frm_var.get(),
                files_found=data_paths,
                clf_name=self.clf_dropdown.getChoices(),
                cores=int(self.multiprocess_dropdown.getChoices()),
                style_attr=style_attr,
            )
        probability_plot_creator.run()


# _ = VisualizeClassificationProbabilityPopUp(config_path='/Users/simon/Desktop/envs/troubleshooting/Two_animals_16bps/project_folder/project_config.ini')



__author__ = "Simon Nilsson"

import multiprocessing
from tkinter import *

from simba.data_processors.timebins_clf_calculator import TimeBinsClfCalculator
from simba.mixins.config_reader import ConfigReader
from simba.mixins.pop_up_mixin import PopUpMixin
from simba.ui.tkinter_functions import CreateLabelFrameWithIcon, Entry_Box
from simba.utils.checks import check_int
from simba.utils.enums import Formats, Keys, Links, Options
from simba.utils.errors import (NoChoosenClassifierError,
                                NoChoosenMeasurementError)


class TimeBinsClfPopUp(PopUpMixin, ConfigReader):
    def __init__(self, config_path: str):
        PopUpMixin.__init__(self, title="CLASSIFICATION BY TIME BINS")
        ConfigReader.__init__(self, config_path=config_path)
        cbox_titles = Options.TIMEBINS_MEASURMENT_OPTIONS.value
        self.timebin_entrybox = Entry_Box(
            self.main_frm, "Set time bin size (s)", "15", validation="numeric"
        )
        measures_frm = CreateLabelFrameWithIcon(
            parent=self.main_frm,
            header="MEASUREMENTS",
            icon_name=Keys.DOCUMENTATION.value,
            icon_link=Links.ANALYZE_ML_RESULTS.value,
        )
        clf_frm = LabelFrame(
            self.main_frm,
            text="CLASSIFIERS",
            font=Formats.LABELFRAME_HEADER_FORMAT.value,
            fg="black",
        )
        self.measurements_var_dict, self.clf_var_dict = {}, {}
        for cnt, title in enumerate(cbox_titles):
            self.measurements_var_dict[title] = BooleanVar()
            cbox = Checkbutton(
                measures_frm, text=title, variable=self.measurements_var_dict[title]
            )
            cbox.grid(row=cnt, sticky=NW)
        for cnt, clf_name in enumerate(self.clf_names):
            self.clf_var_dict[clf_name] = BooleanVar()
            cbox = Checkbutton(
                clf_frm, text=clf_name, variable=self.clf_var_dict[clf_name]
            )
            cbox.grid(row=cnt, sticky=NW)
        run_button = Button(
            self.main_frm, text="Run", command=lambda: self.run_time_bins_clf()
        )
        measures_frm.grid(row=0, sticky=NW)
        clf_frm.grid(row=1, sticky=NW)
        self.timebin_entrybox.grid(row=2, sticky=NW)
        run_button.grid(row=3, sticky=NW)

    def run_time_bins_clf(self):
        check_int(name="Time bin", value=self.timebin_entrybox.entry_get)
        measurement_lst, clf_list = [], []
        for name, val in self.measurements_var_dict.items():
            if val.get():
                measurement_lst.append(name)
        for name, val in self.clf_var_dict.items():
            if val.get():
                clf_list.append(name)
        if len(measurement_lst) == 0:
            raise NoChoosenMeasurementError(source=self.__class__.__name__)
        if len(clf_list) == 0:
            raise NoChoosenClassifierError(source=self.__class__.__name__)
        time_bins_clf_analyzer = TimeBinsClfCalculator(
            config_path=self.config_path,
            bin_length=int(self.timebin_entrybox.entry_get),
            measurements=measurement_lst,
            classifiers=clf_list,
        )

        time_bins_clf_multiprocessor = multiprocessing.Process(
            target=time_bins_clf_analyzer.run()
        )
        time_bins_clf_multiprocessor.start()


# _ = TimeBinsClfPopUp(config_path='/Users/simon/Desktop/envs/troubleshooting/Two_animals_16bps/project_folder/project_config.ini')



__author__ = "Simon Nilsson"

import os
from tkinter import *

from simba.mixins.config_reader import ConfigReader
from simba.mixins.pop_up_mixin import PopUpMixin
from simba.plotting.data_plotter import DataPlotter
from simba.ui.tkinter_functions import CreateLabelFrameWithIcon, DropDownMenu
from simba.utils.enums import Formats, Keys, Links, Paths
from simba.utils.errors import InvalidInputError
from simba.utils.read_write import get_file_name_info_in_directory


class DataPlotterPopUp(PopUpMixin, ConfigReader):
    def __init__(self, config_path: str):

        PopUpMixin.__init__(self, title="CREATE DATA PLOTS")
        ConfigReader.__init__(self, config_path=config_path)
        self.color_lst = list(self.colors_dict.keys())
        self.data_path = os.path.join(self.project_path, Paths.OUTLIER_CORRECTED.value)
        self.files_found_dict = get_file_name_info_in_directory(
            directory=self.data_path, file_type=self.file_type
        )
        self.animal_cnt_options = list(range(1, self.animal_cnt + 1))
        self.style_settings_frm = CreateLabelFrameWithIcon(
            parent=self.main_frm,
            header="STYLE SETTINGS",
            icon_name=Keys.DOCUMENTATION.value,
            icon_link=Links.DATA_TABLES.value,
        )
        self.rounding_decimals_options = list(range(0, 11))
        self.font_thickness_options = list(range(1, 11))
        self.resolution_dropdown = DropDownMenu(
            self.style_settings_frm, "RESOLUTION:", self.resolutions, "18"
        )
        self.rounding_decimals_dropdown = DropDownMenu(
            self.style_settings_frm,
            "DECIMAL ACCURACY:",
            self.rounding_decimals_options,
            "18",
        )
        self.background_color_dropdown = DropDownMenu(
            self.style_settings_frm, "BACKGROUND COLOR: ", self.color_lst, "18"
        )
        self.font_color_dropdown = DropDownMenu(
            self.style_settings_frm, "HEADER COLOR: ", self.color_lst, "18"
        )
        self.font_thickness_dropdown = DropDownMenu(
            self.style_settings_frm,
            "FONT THICKNESS: ",
            self.font_thickness_options,
            "18",
        )

        self.background_color_dropdown.setChoices(choice="White")
        self.font_color_dropdown.setChoices(choice="Black")
        self.resolution_dropdown.setChoices(self.resolutions[1])
        self.rounding_decimals_dropdown.setChoices(2)
        self.font_thickness_dropdown.setChoices(1)

        self.body_parts_frm = LabelFrame(
            self.main_frm,
            text="CHOOSE BODY-PARTS",
            font=Formats.LABELFRAME_HEADER_FORMAT.value,
            pady=5,
            padx=5,
        )
        self.number_of_animals_dropdown = DropDownMenu(
            self.body_parts_frm,
            "# Animals:",
            self.animal_cnt_options,
            "16",
            com=self.__populate_body_parts_menu,
        )
        self.number_of_animals_dropdown.setChoices(self.animal_cnt_options[0])
        self.__populate_body_parts_menu(self.animal_cnt_options[0])

        self.settings_frm = LabelFrame(
            self.main_frm,
            text="VISUALIZATION SETTINGS",
            font=Formats.LABELFRAME_HEADER_FORMAT.value,
            pady=5,
            padx=5,
        )
        self.data_frames_var = BooleanVar()
        self.data_videos_var = BooleanVar()
        data_frames_cb = Checkbutton(
            self.settings_frm, text="Create frames", variable=self.data_frames_var
        )
        data_videos_cb = Checkbutton(
            self.settings_frm, text="Create videos", variable=self.data_videos_var
        )

        self.run_frm = LabelFrame(
            self.main_frm,
            text="RUN",
            font=Formats.LABELFRAME_HEADER_FORMAT.value,
            pady=5,
            padx=5,
            fg="black",
        )
        self.run_single_video_frm = LabelFrame(
            self.run_frm,
            text="SINGLE VIDEO",
            font=Formats.LABELFRAME_HEADER_FORMAT.value,
            pady=5,
            padx=5,
            fg="black",
        )
        self.run_single_video_btn = Button(
            self.run_single_video_frm,
            text="Create single video",
            fg="blue",
            command=lambda: self.__create_data_plots(multiple_videos=False),
        )
        self.single_video_dropdown = DropDownMenu(
            self.run_single_video_frm,
            "Video:",
            list(self.files_found_dict.keys()),
            "12",
        )
        self.single_video_dropdown.setChoices(list(self.files_found_dict.keys())[0])
        self.run_multiple_videos = LabelFrame(
            self.run_frm,
            text="MULTIPLE VIDEO",
            font=Formats.LABELFRAME_HEADER_FORMAT.value,
            pady=5,
            padx=5,
            fg="black",
        )
        self.run_multiple_video_btn = Button(
            self.run_multiple_videos,
            text="Create multiple videos ({} video(s) found)".format(
                str(len(list(self.files_found_dict.keys())))
            ),
            fg="blue",
            command=lambda: self.__create_data_plots(multiple_videos=True),
        )

        self.style_settings_frm.grid(row=0, sticky=NW)
        self.resolution_dropdown.grid(row=0, sticky=NW)
        self.rounding_decimals_dropdown.grid(row=1, sticky=NW)
        self.background_color_dropdown.grid(row=2, sticky=NW)
        self.font_color_dropdown.grid(row=3, sticky=NW)
        self.font_thickness_dropdown.grid(row=4, sticky=NW)

        self.body_parts_frm.grid(row=1, sticky=NW)
        self.number_of_animals_dropdown.grid(row=0, sticky=NW)

        self.settings_frm.grid(row=2, sticky=NW)
        data_frames_cb.grid(row=0, sticky=NW)
        data_videos_cb.grid(row=1, sticky=NW)

        self.run_frm.grid(row=3, sticky=NW)
        self.run_single_video_frm.grid(row=0, sticky=NW)
        self.run_single_video_btn.grid(row=0, sticky=NW)
        self.single_video_dropdown.grid(row=0, column=1, sticky=NW)
        self.run_multiple_videos.grid(row=1, sticky=NW)
        self.run_multiple_video_btn.grid(row=0, sticky=NW)

        self.main_frm.mainloop()

    def __populate_body_parts_menu(self, choice):
        if hasattr(self, "bp_dropdowns"):
            for k, v in self.bp_dropdowns.items():
                self.bp_dropdowns[k].destroy()
                self.bp_colors[k].destroy()
        self.bp_dropdowns, self.bp_colors = {}, {}
        for animal_cnt in range(int(self.number_of_animals_dropdown.getChoices())):
            self.bp_dropdowns[animal_cnt] = DropDownMenu(
                self.body_parts_frm,
                "Body-part {}:".format(str(animal_cnt + 1)),
                self.body_parts_lst,
                "16",
            )
            self.bp_dropdowns[animal_cnt].setChoices(self.body_parts_lst[animal_cnt])
            self.bp_dropdowns[animal_cnt].grid(row=animal_cnt + 1, column=0, sticky=NW)

            self.bp_colors[animal_cnt] = DropDownMenu(
                self.body_parts_frm, "", self.color_lst, "2"
            )
            self.bp_colors[animal_cnt].setChoices(self.color_lst[animal_cnt])
            self.bp_colors[animal_cnt].grid(row=animal_cnt + 1, column=1, sticky=NW)

    def __create_data_plots(self, multiple_videos: bool):

        if multiple_videos:
            data_paths = list(self.files_found_dict.values())
        else:
            data_paths = [
                self.files_found_dict[self.single_video_dropdown.getChoices()]
            ]

        width = int(self.resolution_dropdown.getChoices().split("×")[0])
        height = int(self.resolution_dropdown.getChoices().split("×")[1])
        body_part_attr = []
        for k, v in self.bp_dropdowns.items():
            body_part_attr.append([v.getChoices(), self.bp_colors[k].getChoices()])
        selected_bps = [x[0] for x in body_part_attr]
        if len(list(set(selected_bps))) != len(body_part_attr):
            raise InvalidInputError(
                msg=f"Please choose unique only body-parts for plotting. Got {len(body_part_attr)} body-parts but only {len(list(set(selected_bps)))} unique",
                source=DataPlotterPopUp.__class__.__name__,
            )
        style_attr = {
            "bg_color": self.background_color_dropdown.getChoices(),
            "header_color": self.font_color_dropdown.getChoices(),
            "font_thickness": int(self.font_thickness_dropdown.getChoices()),
            "size": (int(width), int(height)),
            "data_accuracy": int(self.rounding_decimals_dropdown.getChoices()),
        }

        print(data_paths)
        data_plotter = DataPlotter(
            config_path=self.config_path,
            body_part_attr=body_part_attr,
            data_paths=data_paths,
            style_attr=style_attr,
            frame_setting=self.data_frames_var.get(),
            video_setting=self.data_videos_var.get(),
        )

        _ = data_plotter.run()


# _ = DataPlotterPopUp(config_path='/Users/simon/Desktop/envs/troubleshooting/Two_animals_16bps/project_folder/project_config.ini')
# _ = DataPlotterPopUp(config_path='/Users/simon/Desktop/envs/troubleshooting/two_black_animals_14bp/project_folder/project_config.ini')



__author__ = "Simon Nilsson"

import os
import warnings
from copy import deepcopy
from typing import Union

warnings.filterwarnings("ignore", category=FutureWarning)
warnings.filterwarnings("ignore", category=DeprecationWarning)

from simba.mixins.config_reader import ConfigReader
from simba.mixins.train_model_mixin import TrainModelMixin
from simba.utils.checks import check_file_exist_and_readable
from simba.utils.enums import TagNames
from simba.utils.printing import log_event, stdout_success
from simba.utils.read_write import get_fn_ext, read_df, write_df


class InferenceValidation(ConfigReader, TrainModelMixin):
    """
    Run a single classifier on a single featurized input file. Results are saved within the
    ``project_folder/csv/validation`` directory of the SimBA project.

    :param str config_file_path: path to SimBA project config file in Configparser format
    :param str input_file_path: path to file containing features
    :param str clf_path: path to pickled rf sklearn classifier.

    .. note::
       `Tutorial <https://github.com/sgoldenlab/simba/blob/master/docs/Scenario2.md#part-3-run-the-classifier-on-new-data>`_

    :example:
    >>> InferenceValidation(config_path=r"MyProjectConfigPath", input_file_path=r"FeatureFilePath", clf_path=r"ClassifierPath")

    """

    def __init__(
        self,
        config_path: Union[str, os.PathLike],
        input_file_path: Union[str, os.PathLike],
        clf_path: Union[str, os.PathLike],
    ):

        ConfigReader.__init__(self, config_path=config_path)
        TrainModelMixin.__init__(self)
        log_event(
            logger_name=str(self.__class__.__name__),
            log_type=TagNames.CLASS_INIT.value,
            msg=self.create_log_msg_from_init_args(locals=locals()),
        )
        self.save_path = os.path.join(self.project_path, "csv", "validation")
        if not os.path.exists(self.save_path):
            os.makedirs(self.save_path)

        check_file_exist_and_readable(input_file_path)
        check_file_exist_and_readable(clf_path)
        _, file_name, _ = get_fn_ext(str(input_file_path))
        _, classifier_name, _ = get_fn_ext(clf_path)
        data_df = read_df(input_file_path, self.file_type)
        output_df = deepcopy(data_df)
        data_df = self.drop_bp_cords(df=data_df)
        clf = self.read_pickle(file_path=clf_path)
        probability_col_name = f"Probability_{classifier_name}"
        output_df[probability_col_name] = self.clf_predict_proba(
            clf=clf, x_df=data_df, model_name=classifier_name, data_path=input_file_path
        )
        save_filename = os.path.join(self.save_path, file_name + "." + self.file_type)
        write_df(output_df, self.file_type, save_filename)
        self.timer.stop_timer()
        stdout_success(
            msg=f'Validation predictions generated for "{file_name}" within the project_folder/csv/validation directory',
            elapsed_time=self.timer.elapsed_time_str,
            source=self.__class__.__name__,
        )
        print(
            'Click on "Interactive probability plot" to inspect classifier probability thresholds. If satisfactory proceed to specify threshold and minimum bout length and click on "Validate" to create video.'
        )


#
# ValidateModelRunClf(config_path=r"Z:\DeepLabCut\DLC_extract\Troubleshooting\DLC_two_mice\project_folder\project_config.ini",
#                                input_file_path=r"Z:\DeepLabCut\DLC_extract\Troubleshooting\DLC_2_black_060320\project_folder\csv\features_extracted\Together_1.csv",
#                                clf_path=r"Z:\DeepLabCut\DLC_extract\Troubleshooting\DLC_2_black_060320\models\Approach.sav")



from tkinter import *

from simba.mixins.pop_up_mixin import PopUpMixin
from simba.plotting.ez_lineplot import DrawPathPlot
from simba.ui.tkinter_functions import (CreateLabelFrameWithIcon, DropDownMenu,
                                        Entry_Box, FileSelect)
from simba.utils.enums import Keys, Links, Options
from simba.utils.lookups import get_color_dict


class MakePathPlotPopUp(PopUpMixin):
    def __init__(self):
        PopUpMixin.__init__(self, title="CREATE PATH PLOT", size=(500, 300))
        settings_frm = CreateLabelFrameWithIcon(
            parent=self.main_frm,
            header="SETTINGS",
            icon_name=Keys.DOCUMENTATION.value,
            icon_link=Links.VIDEO_TOOLS.value,
        )
        video_path = FileSelect(
            settings_frm,
            "VIDEO PATH: ",
            lblwidth="30",
            file_types=[("VIDEO FILE", Options.ALL_VIDEO_FORMAT_STR_OPTIONS.value)],
        )
        body_part = Entry_Box(settings_frm, "BODY PART: ", "30")
        data_path = FileSelect(
            settings_frm, "DATA PATH (e.g., H5 or CSV file): ", lblwidth="30"
        )
        color_lst = list(get_color_dict().keys())
        background_color = DropDownMenu(
            settings_frm, "BACKGROUND COLOR: ", color_lst, "18"
        )
        background_color.setChoices(choice="White")
        line_color = DropDownMenu(settings_frm, "LINE COLOR: ", color_lst, "18")
        line_color.setChoices(choice="Red")
        line_thickness = DropDownMenu(
            settings_frm, "LINE THICKNESS: ", list(range(1, 11)), "18"
        )
        line_thickness.setChoices(choice=1)
        circle_size = DropDownMenu(
            settings_frm, "CIRCLE SIZE: ", list(range(1, 11)), "18"
        )
        circle_size.setChoices(choice=5)
        run_btn = Button(
            settings_frm,
            text="CREATE PATH PLOT VIDEO",
            command=lambda: DrawPathPlot(
                data_path=data_path.file_path,
                video_path=video_path.file_path,
                body_part=body_part.entry_get,
                bg_color=background_color.getChoices(),
                line_color=line_color.getChoices(),
                line_thinkness=line_thickness.getChoices(),
                circle_size=circle_size.getChoices(),
            ),
        )
        settings_frm.grid(row=0, sticky=W)
        video_path.grid(row=0, sticky=W)
        data_path.grid(row=1, sticky=W)
        body_part.grid(row=2, sticky=W)
        background_color.grid(row=3, sticky=W)
        line_color.grid(row=4, sticky=W)
        line_thickness.grid(row=5, sticky=W)
        circle_size.grid(row=6, sticky=W)
        run_btn.grid(row=7, pady=10)
        # self.main_frm.mainloop()


# MakePathPlotPopUp()



import pytest
import os
from simba.plotting.directing_animals_visualizer import DirectingOtherAnimalsVisualizer
from simba.plotting.directing_animals_visualizer_mp import DirectingOtherAnimalsVisualizerMultiprocess

IN_GITHUB_ACTIONS = os.getenv("GITHUB_ACTIONS") == "true"


class TestDirectingAnimalsVisualizer(object):

    @pytest.fixture(params=['tests/data/test_projects/two_c57/project_folder/csv/outlier_corrected_movement_location/Together_1.csv'])
    def data_path_args(self, request):
        return request

    @pytest.fixture(params=['tests/data/test_projects/two_c57/project_folder/project_config.ini'])
    def config_path_args(self, request):
        return request

    @pytest.fixture(params=[True, False])
    def show_pose_args(self, request):
        return request

    @pytest.fixture(params=[3])
    def circle_size_args(self, request):
        return request

    @pytest.fixture(params=[5])
    def core_cnt_args(self, request):
        return request

    @pytest.fixture(params=['Random', 'Orange'])
    def direction_color_args(self, request):
        return request

    @pytest.fixture(params=[4])
    def direction_thickness_args(self, request):
        return request

    @pytest.fixture(params=[True, False])
    def highlight_endpoints_args(self, request):
        return request

    @pytest.fixture(params=[True, False])
    def polyfill_args(self, request):
        return request

    def test_directing_animal_visualizer_single_core(self,
                                                     data_path_args,
                                                     config_path_args,
                                                     show_pose_args,
                                                     circle_size_args,
                                                     direction_color_args,
                                                     direction_thickness_args,
                                                     highlight_endpoints_args,
                                                     polyfill_args):

        style_attr = {'Show_pose': show_pose_args.param,
                      'Pose_circle_size': circle_size_args.param,
                      "Direction_color": direction_color_args.param,
                      'Direction_thickness': direction_thickness_args.param,
                      'Highlight_endpoints': highlight_endpoints_args.param,
                      'Polyfill': polyfill_args.param}

        single_core_visualizer = DirectingOtherAnimalsVisualizer(config_path=config_path_args.param,
                                                                 style_attr=style_attr,
                                                                 data_path=data_path_args.param)
        single_core_visualizer.run()
    
    
    @pytest.mark.skipif(IN_GITHUB_ACTIONS, reason="LONG RUNNING TIME.")
    def test_directing_animal_visualizer_multi_core(self,
                                                    data_path_args,
                                                    core_cnt_args,
                                                    config_path_args,
                                                    show_pose_args,
                                                    circle_size_args,
                                                    direction_color_args,
                                                    direction_thickness_args,
                                                    highlight_endpoints_args,
                                                    polyfill_args):

        style_attr = {'Show_pose': show_pose_args.param,
                      'Pose_circle_size': circle_size_args.param,
                      "Direction_color": direction_color_args.param,
                      'Direction_thickness': direction_thickness_args.param,
                      'Highlight_endpoints': highlight_endpoints_args.param,
                      'Polyfill': polyfill_args.param}

        multi_core_visualizer = DirectingOtherAnimalsVisualizerMultiprocess(config_path=config_path_args.param,
                                                                            style_attr=style_attr,
                                                                            data_path=data_path_args.param,
                                                                            core_cnt=core_cnt_args.param)
        multi_core_visualizer.run()



import os
from tkinter import *
from tkinter import filedialog
from typing import List, Union

from simba.mixins.annotator_mixin import AnnotatorMixin
from simba.mixins.config_reader import ConfigReader
from simba.utils.checks import check_file_exist_and_readable
from simba.utils.enums import Options
from simba.utils.read_write import (find_video_of_file, get_fn_ext,
                                    get_video_meta_data)


class TargetedAnnotatorWithClips(AnnotatorMixin, ConfigReader):
    def __init__(
        self, config_path: Union[str, os.PathLike], video_path: Union[str, os.PathLike]
    ):
        ConfigReader.__init__(self, config_path=config_path)
        _, video_name, _ = get_fn_ext(filepath=video_path)
        data_path = os.path.join(
            self.machine_results_dir, f"{video_name}.{self.file_type}"
        )
        AnnotatorMixin.__init__(
            self,
            config_path=config_path,
            video_path=video_path,
            data_path=data_path,
            frame_size=(700, 500),
        )

        self.video_frm_label(frm_number=self.min_frm_no)
        self.nav_bar = self.h_nav_bar(
            parent=self.main_frm, change_frm_func=self.change_frame_targeted_annotations
        )
        self.selection_pane = self.targeted_clips_pane(parent=self.main_frm)
        self.play_video_frame = self.v_navigation_pane_targeted_clips_version(
            parent=self.main_frm
        )

        self.nav_bar.grid(row=1, column=0, sticky=NW)
        self.selection_pane.grid(row=0, column=1, sticky=NW)
        self.play_video_frame.grid(row=0, column=2, sticky=NW)
        self.main_frm.mainloop()


def select_labelling_video_targeted_clips(config_path: Union[str, os.PathLike]):
    video_file_path = filedialog.askopenfilename(
        filetypes=[("Video files", Options.ALL_VIDEO_FORMAT_STR_OPTIONS.value)]
    )
    check_file_exist_and_readable(video_file_path)
    video_meta = get_video_meta_data(video_file_path)
    _, video_name, _ = get_fn_ext(video_file_path)
    print(f"ANNOTATING VIDEO {video_name} \n  VIDEO INFO: {video_meta}")
    _ = TargetedAnnotatorWithClips(config_path=config_path, video_path=video_file_path)


# test = TargetedAnnotatorWithClips(config_path='/Users/simon/Desktop/envs/troubleshooting/two_black_animals_14bp/project_folder/project_config.ini', video_name='Together_1')



__author__ = "Simon Nilsson"

import warnings

warnings.simplefilter(action="ignore", category=FutureWarning)
import functools
import multiprocessing
import os
import platform
import shutil
from typing import Dict, List

import cv2
import numpy as np

from simba.mixins.config_reader import ConfigReader
from simba.mixins.plotting_mixin import PlottingMixin
from simba.utils.checks import check_if_filepath_list_is_empty
from simba.utils.data import detect_bouts
from simba.utils.enums import Formats
from simba.utils.errors import NoSpecifiedOutputError
from simba.utils.lookups import get_named_colors
from simba.utils.printing import SimbaTimer, stdout_success
from simba.utils.read_write import (concatenate_videos_in_folder, get_fn_ext,
                                    read_df)


class GanttCreatorMultiprocess(ConfigReader, PlottingMixin):
    """
    Multiprocess creation of classifier gantt charts in video and/or image format.
    See meth:`simba.gantt_creator.GanttCreatorSingleProcess` for single-process class.

    ..note::
       `GitHub gantt tutorial <https://github.com/sgoldenlab/simba/blob/master/docs/tutorial.md#gantt-plot>`__.

    .. image:: _static/img/gantt_plot.png
       :width: 300
       :align: center

    :param str config_path: path to SimBA project config file in Configparser format.
    :param bool frame_setting: If True, creates individual frames.
    :param bool last_frm_setting: If True, creates single .png image representing entire video.
    :param bool video_setting: If True, creates videos
    :param dict style_attr: Attributes of gannt chart (size, font size, font rotation etc).
    :param List[str] files_found: File paths representing files with machine predictions e.g., ['project_folder/csv/machine_results/My_results.csv']
    :param int cores: Number of cores to use.

    Examples
    ----------
    >>> gantt_creator = GanttCreatorMultiprocess(config_path='project_folder/project_config.ini', frame_setting=False, video_setting=True, files_found=['project_folder/csv/machine_results/Together_1.csv'], cores=5, style_attr={'width': 640, 'height': 480, 'font size': 8, 'font rotation': 45}).run()

    """

    def __init__(
        self,
        config_path: str,
        frame_setting: bool,
        video_setting: bool,
        files_found: List[str],
        cores: int,
        style_attr: Dict[str, int],
        last_frm_setting: bool,
    ):
        if platform.system() == "Darwin":
            multiprocessing.set_start_method("spawn", force=True)

        ConfigReader.__init__(self, config_path=config_path)
        PlottingMixin.__init__(self)

        (
            self.frame_setting,
            self.video_setting,
            self.files_found,
            self.style_attr,
            self.cores,
            self.last_frm_setting,
        ) = (
            frame_setting,
            video_setting,
            files_found,
            style_attr,
            cores,
            last_frm_setting,
        )
        if (
            (not self.frame_setting)
            and (not self.video_setting)
            and (not self.last_frm_setting)
        ):
            raise NoSpecifiedOutputError(
                msg="SIMBA ERROR: Please select gantt videos, frames, and/or last frame."
            )
        check_if_filepath_list_is_empty(
            filepaths=self.files_found,
            error_msg="SIMBA ERROR: Zero files found in the project_folder/csv/machine_results directory. Create classification results before visualizing gantt charts",
        )
        self.colours = get_named_colors()[:-1]
        self.colour_tuple_x = list(np.arange(3.5, 203.5, 5))
        if not os.path.exists(self.gantt_plot_dir):
            os.makedirs(self.gantt_plot_dir)
        self.y_rotation, self.y_fontsize = (
            self.style_attr["font rotation"],
            self.style_attr["font size"],
        )
        self.fourcc = cv2.VideoWriter_fourcc(*Formats.MP4_CODEC.value)
        self.out_width, self.out_height = (
            self.style_attr["width"],
            self.style_attr["height"],
        )
        print("Processing {} video(s)...".format(str(len(self.files_found))))

    def run(self):
        """
        Creates gantt charts. Results are stored in the `project_folder/frames/gantt_plots` directory of SimBA project.

        Returns
        ----------
        None
        """

        for file_cnt, file_path in enumerate(self.files_found):
            video_timer = SimbaTimer()
            video_timer.start_timer()
            _, self.video_name, _ = get_fn_ext(file_path)
            self.data_df = read_df(file_path, self.file_type).reset_index(drop=True)
            print(
                "Processing video {}, Frame count: {} (Video {}/{})...".format(
                    self.video_name,
                    str(len(self.data_df)),
                    str(file_cnt + 1),
                    str(len(self.files_found)),
                )
            )
            self.video_info_settings, _, self.fps = self.read_video_info(
                video_name=self.video_name
            )
            self.bouts_df = detect_bouts(
                data_df=self.data_df, target_lst=list(self.clf_names), fps=int(self.fps)
            )
            self.temp_folder = os.path.join(
                self.gantt_plot_dir, self.video_name, "temp"
            )
            self.save_frame_folder_dir = os.path.join(
                self.gantt_plot_dir, self.video_name
            )
            if self.frame_setting:
                if os.path.exists(self.save_frame_folder_dir):
                    shutil.rmtree(self.save_frame_folder_dir)
                if not os.path.exists(self.save_frame_folder_dir):
                    os.makedirs(self.save_frame_folder_dir)
            if self.video_setting:
                self.video_folder = os.path.join(self.gantt_plot_dir, self.video_name)
                if os.path.exists(self.temp_folder):
                    shutil.rmtree(self.temp_folder)
                    shutil.rmtree(self.video_folder)
                os.makedirs(self.temp_folder)
                self.save_video_path = os.path.join(
                    self.gantt_plot_dir, self.video_name + ".mp4"
                )

            if self.last_frm_setting:
                _ = self.make_gantt_plot(
                    data_df=self.data_df,
                    bouts_df=self.bouts_df,
                    clf_names=self.clf_names,
                    fps=self.fps,
                    style_attr=self.style_attr,
                    video_name=self.video_name,
                    save_path=os.path.join(
                        self.gantt_plot_dir, self.video_name + "_final_image.png"
                    ),
                )

            if self.video_setting or self.frame_setting:
                frame_array = np.array_split(
                    list(range(0, len(self.data_df))), self.cores
                )
                frm_per_core = len(frame_array[0])
                for group_cnt, rng in enumerate(frame_array):
                    frame_array[group_cnt] = np.insert(rng, 0, group_cnt)

                print(
                    "Creating gantt, multiprocessing (chunksize: {}, cores: {})...".format(
                        str(self.multiprocess_chunksize), str(self.cores)
                    )
                )
                with multiprocessing.Pool(
                    self.cores, maxtasksperchild=self.maxtasksperchild
                ) as pool:
                    constants = functools.partial(
                        self.gantt_creator_mp,
                        video_setting=self.video_setting,
                        frame_setting=self.frame_setting,
                        video_save_dir=self.temp_folder,
                        frame_folder_dir=self.save_frame_folder_dir,
                        bouts_df=self.bouts_df,
                        rotation=self.y_rotation,
                        clf_names=self.clf_names,
                        colors=self.colours,
                        color_tuple=self.colour_tuple_x,
                        fps=self.fps,
                        font_size=self.y_fontsize,
                        width=self.out_width,
                        height=self.out_height,
                        video_name=self.video_name,
                    )

                    for cnt, result in enumerate(
                        pool.imap(
                            constants,
                            frame_array,
                            chunksize=self.multiprocess_chunksize,
                        )
                    ):
                        print(
                            "Image {}/{}, Video {}/{}...".format(
                                str(int(frm_per_core * (result + 1))),
                                str(len(self.data_df)),
                                str(file_cnt + 1),
                                str(len(self.files_found)),
                            )
                        )
                    pool.terminate()
                    pool.join()

                if self.video_setting:
                    print("Joining {} multiprocessed video...".format(self.video_name))
                    concatenate_videos_in_folder(
                        in_folder=self.temp_folder, save_path=self.save_video_path
                    )
                video_timer.stop_timer()
                print(
                    "Gantt video {} complete (elapsed time: {}s) ...".format(
                        self.video_name, video_timer.elapsed_time_str
                    )
                )

        self.timer.stop_timer()
        stdout_success(
            msg=f"Gantt visualizations for {len(self.files_found)} videos created in project_folder/frames/output/gantt_plots directory",
            elapsed_time=self.timer.elapsed_time_str,
        )


# test = GanttCreatorMultiprocess(config_path='/Users/simon/Desktop/envs/troubleshooting/two_black_animals_14bp/project_folder/project_config.ini',
#                                 frame_setting=False,
#                                 video_setting=True,
#                                 files_found=['/Users/simon/Desktop/envs/troubleshooting/two_black_animals_14bp/project_folder/csv/targets_inserted/Together_1.csv'],
#                                 cores=5,
#                                 last_frm_setting=False,
#                                 style_attr={'width': 640, 'height': 480, 'font size': 12, 'font rotation': 45})
# test.run()


# style_attr = {'width': 640, 'height': 480, 'font size': 12, 'font rotation': 45}
# test = GanttCreatorMultiprocess(config_path='/Users/simon/Desktop/envs/troubleshooting/two_black_animals/project_folder/project_config.ini',
#                                  frame_setting=False,
#                                  video_setting=True,
#                                  last_frm_setting=False,
#                                  style_attr=style_attr,
# cores=5,
#                                  files_found=['/Users/simon/Desktop/envs/troubleshooting/two_black_animals/project_folder/csv/machine_results/Together_1.csv'])
# test.create_gannt()



__author__ = "Simon Nilsson"

import os
from collections import defaultdict
from copy import deepcopy

import pandas as pd
from joblib import Parallel, delayed
from shapely.geometry import Point

from simba.mixins.config_reader import ConfigReader
from simba.utils.enums import Formats
from simba.utils.errors import NoFilesFoundError
from simba.utils.printing import stdout_success
from simba.utils.read_write import read_df, write_df


class BoundaryStatisticsCalculator(ConfigReader):
    """
    Compute boundary intersection statistics.

    :parameter str config_path: Path to SimBA project config file in Configparser format.
    :parameter bool roi_intersections: If True, calculates intersection of animal-anchored ROIs
    :parameter bool roi_keypoint_intersections: If True, calculates intersection of animal-anchored ROIs and pose-estimated animal key-points.
    :parameter str save_format: Output data format. OPTIONS: CSV, PARQUET, PICKLE.

    Notes
    ----------
    `Bounding boxes tutorial <https://github.com/sgoldenlab/simba/blob/master/docs/anchored_rois.md>`_.

    Examples
    ----------
    >>> boundary_stats_calculator = BoundaryStatisticsCalculator(config_path='MyConfigFile',roi_intersections=True, roi_keypoint_intersections=True, save_format='CSV')
    >>> boundary_stats_calculator.save_results()
    """

    def __init__(
        self,
        config_path: str,
        roi_intersections: bool,
        roi_keypoint_intersections: bool,
        save_format: str,
    ):
        ConfigReader.__init__(self, config_path=config_path)
        self.save_format, self.roi_intersections, self.roi_keypoint_intersections = (
            save_format,
            roi_intersections,
            roi_keypoint_intersections,
        )
        self.anchored_roi_path = os.path.join(
            self.project_path, "logs", "anchored_rois.pickle"
        )
        self.save_folder = os.path.join(self.project_path, "csv", "anchored_roi_data")
        if not os.path.isfile(self.anchored_roi_path):
            raise NoFilesFoundError(
                msg=f"No anchored ROI data detected. Extract anchored ROIs before computing statistics. File expected at path {self.anchored_roi_path}"
            )
        self.polygons = read_df(
            file_path=self.anchored_roi_path, file_type=Formats.PICKLE.value
        )
        self.calculate_statistics()

    def _find_intersections(self, animal_roi: list, other_animals: dict):
        results = []
        for first_animal, second_animal in zip(animal_roi, other_animals):
            results.append(first_animal.intersects(second_animal))
        return results

    def _find_points_in_roi(self, animal_roi: list, second_animal_bps: list):
        results = []
        for polygon_cnt, polygon in enumerate(animal_roi):
            frm_results = []
            for k, v in second_animal_bps[polygon_cnt].items():
                frm_results.append(Point(v).within(polygon))
            results.append(frm_results)
        return results

    def _sort_keypoint_results(self, first_animal_name: str, second_animal_name: str):
        results = defaultdict(list)
        for frm in self.results:
            for body_part_cnt, body_part in enumerate(frm):
                body_part_name = self.animal_bp_dict[second_animal_name]["X_bps"][
                    body_part_cnt
                ][:-4]
                results[
                    "{}:{}:{}".format(
                        first_animal_name, second_animal_name, body_part_name
                    )
                ].append(int(body_part))
        return pd.DataFrame(results)

    def _sort_intersection_results(self):
        results = defaultdict(list)
        for animal_one_name, animal_one_data in self.intersecting_rois.items():
            for animal_two_name, animal_two_data in animal_one_data.items():
                results["{}:{}:ROI_ONLY".format(animal_one_name, animal_two_name)] = [
                    int(x) for x in animal_two_data
                ]
        return pd.DataFrame(results)

    def calculate_statistics(self):
        self.intersection_dfs = {}
        self.keypoint_dfs = {}
        for video_cnt, (video_name, video_data) in enumerate(self.polygons.items()):
            print("Calculating statistics for video {}...".format(video_name))
            if self.roi_intersections:
                self.intersecting_rois = {}
                print(
                    "Calculating intersecting anchored ROIs for video {}...".format(
                        video_name
                    )
                )
                for first_animal in self.animal_bp_dict.keys():
                    first_animal_anchored_rois = [
                        video_data[first_animal][i : i + 100]
                        for i in range(0, len(video_data[first_animal]), 100)
                    ]
                    self.intersecting_rois[first_animal] = {}
                    for second_animal in {
                        k: v for k, v in video_data.items() if k != first_animal
                    }.keys():
                        second_animal_anchored_rois = [
                            video_data[second_animal][i : i + 100]
                            for i in range(0, len(video_data[second_animal]), 100)
                        ]
                        results = Parallel(n_jobs=5, verbose=2, backend="threading")(
                            delayed(self._find_intersections)(i, j)
                            for i, j in zip(
                                first_animal_anchored_rois, second_animal_anchored_rois
                            )
                        )
                        self.intersecting_rois[first_animal][second_animal] = [
                            i for s in results for i in s
                        ]
                self.intersection_dfs[video_name] = self._sort_intersection_results()

            if self.roi_keypoint_intersections:
                self.data_df = read_df(
                    os.path.join(
                        self.outlier_corrected_dir, video_name + "." + self.file_type
                    ),
                    self.file_type,
                ).astype(int)
                keypoints_df_lst = []
                print(
                    "Calculate intersecting anchored ROIs and keypoints for video {}...".format(
                        video_name
                    )
                )
                for first_animal in self.animal_bp_dict.keys():
                    first_animal_anchored_rois = [
                        video_data[first_animal][i : i + 100]
                        for i in range(0, len(video_data[first_animal]), 100)
                    ]
                    for second_animal in {
                        k: v
                        for k, v in self.animal_bp_dict.items()
                        if k != first_animal
                    }.keys():
                        second_animal_name = deepcopy(second_animal)
                        second_animal_df_tuples = pd.DataFrame()
                        for x_col, y_col in zip(
                            self.animal_bp_dict[second_animal]["X_bps"],
                            self.animal_bp_dict[second_animal]["Y_bps"],
                        ):
                            second_animal_df_tuples[x_col[:-4]] = list(
                                zip(self.data_df[x_col], self.data_df[y_col])
                            )
                        second_animal = second_animal_df_tuples.to_dict(
                            orient="records"
                        )
                        second_animal = [
                            second_animal[i : i + 100]
                            for i in range(0, len(second_animal), 100)
                        ]
                        results = Parallel(n_jobs=5, verbose=1, backend="threading")(
                            delayed(self._find_points_in_roi)(i, j)
                            for i, j in zip(first_animal_anchored_rois, second_animal)
                        )
                        self.results = [i for s in results for i in s]
                        keypoints_df_lst.append(
                            self._sort_keypoint_results(
                                first_animal_name=first_animal,
                                second_animal_name=second_animal_name,
                            )
                        )
                self.keypoint_dfs[video_name] = pd.concat(keypoints_df_lst, axis=1)

    def save_results(self):
        if not os.path.exists(self.save_folder):
            os.makedirs(self.save_folder)
        out_df = None
        for video_cnt, (video_name, video_data) in enumerate(self.polygons.items()):
            save_path = os.path.join(
                self.save_folder, f"{video_name}.{self.save_format.lower()}"
            )
            if (self.roi_intersections) and (self.roi_keypoint_intersections):
                out_df = pd.concat(
                    [self.intersection_dfs[video_name], self.keypoint_dfs[video_name]],
                    axis=1,
                )
            elif self.roi_intersections:
                out_df = self.intersection_dfs[video_name]
            elif self.roi_keypoint_intersections:
                out_df = self.keypoint_dfs[video_name]
            write_df(df=out_df, file_type=self.save_format.lower(), save_path=save_path)
            print(f"Data for video {video_name} saved...")

        stdout_success(
            msg=f"Data for {str(len(self.polygons.keys()))} videos saved in {self.save_folder}"
        )


# boundary_stats_calculator = BoundaryStatisticsCalculator(config_path='/Users/simon/Desktop/envs/troubleshooting/sleap_5_animals/project_folder/project_config.ini',
#                                                          roi_intersections=True,
#                                                          roi_keypoint_intersections=True,
#                                                          save_format='PICKLE')
# boundary_stats_calculator.save_results()



__author__ = "Simon Nilsson"

import os
from copy import deepcopy

import cv2
import numpy as np
import pandas as pd

from simba.utils.errors import (DataHeaderError, DuplicationError,
                                InvalidFileTypeError)
from simba.utils.lookups import get_color_dict
from simba.utils.printing import SimbaTimer, stdout_success
from simba.utils.read_write import (get_fn_ext, get_video_meta_data,
                                    read_config_file, read_df)


class DrawPathPlot(object):
    def __init__(
        self,
        data_path: str,
        video_path: str,
        body_part: str,
        bg_color: str,
        line_color: str,
        line_thinkness: int,
        circle_size: int,
    ):
        self.timer = SimbaTimer(start=True)
        self.named_shape_colors = get_color_dict()
        self.line_clr_bgr = self.named_shape_colors[line_color]
        self.bg_clr_bgr = self.named_shape_colors[bg_color]
        self.line_thinkness, self.circle_size = int(line_thinkness), int(circle_size)
        if self.line_clr_bgr == self.bg_clr_bgr:
            raise DuplicationError(
                msg="The line color and background color are identical"
            )
        directory, file_name, ext = get_fn_ext(filepath=data_path)
        if ext.lower() == ".h5":
            self.data = pd.read_hdf(data_path)
            headers = []
            if len(self.data.columns[0]) == 4:
                for c in self.data.columns:
                    headers.append("{}_{}_{}".format(c[1], c[2], c[3]))
            if len(self.data.columns[0]) == 3:
                for c in self.data.columns:
                    headers.append("{}_{}".format(c[2], c[3]))
            self.data.columns = headers
            if len(self.data.columns[0]) == 4:
                self.data = self.data.loc[3:]
            if len(self.data.columns[0]) == 3:
                self.data = self.data.loc[2:]
        elif ext.lower() == ".csv":
            self.data = pd.read_csv(data_path)
        else:
            raise InvalidFileTypeError(
                msg=f"File type {str(ext)} is not supported (OPTIONS: h5 or csv)"
            )
        body_parts_available = list(set([x[:-2] for x in self.data.columns]))
        self.col_heads = [body_part + "_x", body_part + "_y", body_part + "_likelihood"]
        if self.col_heads[0] not in self.data.columns:
            raise DataHeaderError(
                msg=f"Body-part {body_part} is not present in the data file. The body-parts available are: {body_parts_available}"
            )
        self.data = (
            self.data[self.col_heads]
            .fillna(method="ffill")
            .astype(int)
            .reset_index(drop=True)
        )
        self.video_meta_data = get_video_meta_data(video_path=video_path)
        self.save_name = os.path.join(
            os.path.dirname(video_path), file_name + "_line_plot.mp4"
        )
        self.bg_image = np.zeros(
            [self.video_meta_data["height"], self.video_meta_data["width"], 3]
        )
        self.bg_image[:] = self.named_shape_colors[bg_color]
        self.bg_image = np.uint8(self.bg_image)
        self.writer = cv2.VideoWriter(
            self.save_name,
            0x7634706D,
            int(self.video_meta_data["fps"]),
            (self.video_meta_data["width"], self.video_meta_data["height"]),
        )
        self.cap = cv2.VideoCapture(video_path)
        self.draw_video()

    def draw_video(self):
        frm_counter, prior_x, prior_y = 0, 0, 0
        while self.cap.isOpened():
            ret, frame = self.cap.read()
            if ret == True:
                current_x, current_y = (
                    self.data.loc[frm_counter, self.col_heads[0]],
                    self.data.loc[frm_counter, self.col_heads[1]],
                )
                if frm_counter > 0:
                    cv2.line(
                        self.bg_image,
                        (prior_x, prior_y),
                        (current_x, current_y),
                        self.line_clr_bgr,
                        self.line_thinkness,
                    )
                prior_x, prior_y = deepcopy(current_x), deepcopy(current_y)
                output_frm = deepcopy(self.bg_image)
                cv2.circle(
                    output_frm,
                    (current_x, current_y),
                    self.circle_size,
                    self.line_clr_bgr,
                    -1,
                )
                self.writer.write(output_frm)
                frm_counter += 1
                print(
                    "Frame {}/{}".format(
                        str(frm_counter), str(self.video_meta_data["frame_count"])
                    )
                )

            else:
                break

        self.cap.release()
        self.timer.stop_timer()
        stdout_success(
            msg=f"Path plot saved at {self.save_name}",
            elapsed_time=self.timer.elapsed_time_str,
        )


def draw_line_plot(configini, video, bodypart):
    configFile = str(configini)
    config = read_config_file(configini)
    configdir = os.path.dirname(configini)
    wfileType = "csv"
    dir_path, vid_name, ext = get_fn_ext(video)
    csvname = vid_name + "." + wfileType
    tracking_csv = os.path.join(
        configdir, "csv", "outlier_corrected_movement_location", csvname
    )
    inputDf = read_df(tracking_csv, wfileType)
    videopath = os.path.join(configdir, "videos", video)
    outputvideopath = os.path.join(configdir, "frames", "output", "simple_path_plots")

    if not os.path.exists(outputvideopath):
        os.mkdir(outputvideopath)

    # datacleaning
    colHeads = [bodypart + "_x", bodypart + "_y", bodypart + "_p"]
    df = inputDf[colHeads].copy()

    widthlist = df[colHeads[0]].astype(float).astype(int)
    heightlist = df[colHeads[1]].astype(float).astype(int)
    circletup = tuple(zip(widthlist, heightlist))

    # get resolution of video
    vcap = cv2.VideoCapture(videopath)
    if vcap.isOpened():
        width = int(vcap.get(cv2.CAP_PROP_FRAME_WIDTH))  # float
        height = int(vcap.get(cv2.CAP_PROP_FRAME_HEIGHT))  # float
        fps = int(vcap.get(cv2.CAP_PROP_FPS))
        totalFrameCount = int(vcap.get(cv2.CAP_PROP_FRAME_COUNT))

    # make white background
    img = np.zeros([height, width, 3])
    img.fill(255)
    img = np.uint8(img)

    fourcc = cv2.VideoWriter_fourcc(*"MJPG")
    out = cv2.VideoWriter(
        os.path.join(outputvideopath, video), 0x7634706D, fps, (width, height)
    )
    counter = 0
    while vcap.isOpened():
        ret, frame = vcap.read()
        if ret == True:
            if counter != 0:
                cv2.line(img, circletup[counter - 1], circletup[counter], 5)

            lineWithCircle = img.copy()
            cv2.circle(lineWithCircle, circletup[counter], 5, [0, 0, 255], -1)

            out.write(lineWithCircle)
            counter += 1
            print("Frame " + str(counter) + "/" + str(totalFrameCount))

        else:
            break

    vcap.release()
    cv2.destroyAllWindows()
    print("Video generated.")


def draw_line_plot_tools(videopath, csvfile, bodypart):
    inputDf = pd.read_csv(csvfile)

    # restructure
    col1 = inputDf.loc[0].to_list()
    col2 = inputDf.loc[1].to_list()
    finalcol = [m + "_" + n for m, n in zip(col1, col2)]
    inputDf.columns = finalcol
    inputDf = inputDf.loc[2:]
    print(inputDf.columns)
    inputDf = inputDf.reset_index(drop=True)
    # datacleaning
    colHeads = [bodypart + "_x", bodypart + "_y", bodypart + "_likelihood"]
    df = inputDf[colHeads].copy()

    widthlist = df[colHeads[0]].astype(float).astype(int)
    heightlist = df[colHeads[1]].astype(float).astype(int)
    circletup = tuple(zip(widthlist, heightlist))

    # get resolution of video
    vcap = cv2.VideoCapture(videopath)
    if vcap.isOpened():
        width = int(vcap.get(cv2.CAP_PROP_FRAME_WIDTH))  # float
        height = int(vcap.get(cv2.CAP_PROP_FRAME_HEIGHT))  # float
        fps = int(vcap.get(cv2.CAP_PROP_FPS))
        totalFrameCount = int(vcap.get(cv2.CAP_PROP_FRAME_COUNT))

    # make white background
    img = np.zeros([height, width, 3])
    img.fill(255)
    img = np.uint8(img)

    outputvideoname = os.path.join(
        os.path.dirname(videopath), "line_plot" + os.path.basename(videopath)
    )
    fourcc = cv2.VideoWriter_fourcc(*"MJPG")
    out = cv2.VideoWriter(outputvideoname, 0x7634706D, fps, (width, height))
    counter = 0
    while vcap.isOpened():
        ret, frame = vcap.read()
        if ret == True:
            if counter != 0:
                cv2.line(img, circletup[counter - 1], circletup[counter], 5)

            lineWithCircle = img.copy()
            cv2.circle(lineWithCircle, circletup[counter], 5, [0, 0, 255], -1)

            out.write(lineWithCircle)
            counter += 1
            print("Frame " + str(counter) + "/" + str(totalFrameCount))

        else:
            break

    vcap.release()
    cv2.destroyAllWindows()
    stdout_success(msg="Video generated.")



import os
from tkinter import *

from simba.data_processors.directing_other_animals_calculator import \
    DirectingOtherAnimalsAnalyzer
from simba.mixins.config_reader import ConfigReader
from simba.mixins.pop_up_mixin import PopUpMixin
from simba.ui.tkinter_functions import CreateLabelFrameWithIcon
from simba.utils.enums import Keys, Links
from simba.utils.errors import (AnimalNumberError, InvalidInputError,
                                NoFilesFoundError)


class AnimalDirectingAnimalPopUp(ConfigReader, PopUpMixin):
    def __init__(self, config_path: str):
        ConfigReader.__init__(self, config_path=config_path)
        if self.animal_cnt < 2:
            raise AnimalNumberError(
                msg=f"Directionality between animals require at least two animals. The SimBA project is set to use {self.animal_cnt} animal.",
                source=self.__class__.__name__,
            )
        if len(self.outlier_corrected_paths) == 0:
            raise NoFilesFoundError(
                msg=f"No data files found in {self.outlier_corrected_dir}.",
                source=self.__class__.__name__,
            )
        PopUpMixin.__init__(
            self, title="ANALYZE DIRECTIONALITY BETWEEN ANIMALS", size=(400, 400)
        )
        self.settings_frm = CreateLabelFrameWithIcon(
            parent=self.main_frm,
            header="OUTPUT FORMATS",
            icon_name=Keys.DOCUMENTATION.value,
            icon_link=Links.DIRECTING_ANIMALS_PLOTS.value,
        )
        self.boolean_tables_var = BooleanVar(value=False)
        self.summary_tables_var = BooleanVar(value=False)
        self.aggregate_statistics_var = BooleanVar(value=False)
        boolean_tables_cb = Checkbutton(
            self.settings_frm,
            text="CREATE BOOLEAN TABLES",
            variable=self.boolean_tables_var,
        )
        summary_table_cb = Checkbutton(
            self.settings_frm,
            text="CREATE DETAILED SUMMARY TABLES (INCLUDING COORDINATES)",
            variable=self.summary_tables_var,
        )
        aggregate_statistics_cb = Checkbutton(
            self.settings_frm,
            text="CREATE AGGREGATE STATISTICS TABLE",
            variable=self.aggregate_statistics_var,
        )

        self.settings_frm.grid(row=0, column=0, sticky="NW")
        boolean_tables_cb.grid(row=0, column=0, sticky="NW")
        summary_table_cb.grid(row=1, column=0, sticky="NW")
        aggregate_statistics_cb.grid(row=2, column=0, sticky="NW")
        self.create_run_frm(run_function=self.run)
        # self.main_frm.mainloop()

    def run(self):
        if (
            not self.boolean_tables_var.get()
            and not self.summary_tables_var.get()
            and not self.aggregate_statistics_var.get()
        ):
            raise InvalidInputError(
                "Please select at least one output format.",
                source=self.__class__.__name__,
            )

        directing_animals_analyzer = DirectingOtherAnimalsAnalyzer(
            config_path=self.config_path,
            bool_tables=self.boolean_tables_var.get(),
            summary_tables=self.summary_tables_var.get(),
            aggregate_statistics_tables=self.aggregate_statistics_var.get(),
        )
        directing_animals_analyzer.run()


# test = AnimalDirectingAnimalPopUp(config_path='/Users/simon/Desktop/envs/troubleshooting/two_black_animals_14bp/project_folder/project_config.ini')



import os
from typing import Optional, Tuple, Union

import cv2
import numpy as np

from simba.utils.checks import (check_file_exist_and_readable,
                                check_if_valid_rgb_tuple, check_int, check_str)
from simba.utils.enums import Options
from simba.utils.errors import InvalidFileTypeError
from simba.utils.read_write import get_fn_ext, get_video_meta_data
from simba.utils.warnings import CropWarning


class ROISelector:
    """
    A class for selecting and reflecting Regions of Interest (ROI) in an image.
    The selected region variables:  top_left, bottom_right, width, height are stored in self.

    :param Union[str, os.PathLike] path: Path to the image or video file. Can also be an image represented as a numpy array.
    :param int thickness: Thickness of the rectangle border for visualizing the ROI.
    :param Tuple[int, int, int] clr: BGR color tuple for visualizing the ROI. Default: deep pink.
    :param Optional[str] title: Title of the drawing window. If None, then `Draw ROI - Press ESC when drawn`.

    :example:
    >>> img_selector = ROISelector(path='/Users/simon/Desktop/compute_overlap.png', clr=(0, 255, 0), thickness=2)
    >>> img_selector.run()
    """

    def __init__(
        self,
        path: Union[str, os.PathLike],
        thickness: int = 10,
        clr: Tuple[int, int, int] = (147, 20, 255),
        title: Optional[str] = None,
    ) -> None:
        check_file_exist_and_readable(file_path=path)
        check_if_valid_rgb_tuple(data=clr)
        check_int(name="Thickness", value=thickness, min_value=1, raise_error=True)
        if title is not None:
            check_str(name="ROI title window", value=title)
        self.roi_start = None
        self.roi_end = None
        self.selecting_roi = False
        self.clr = clr
        self.thickness = int(thickness)

        if isinstance(path, np.ndarray):
            self.image = path
        else:
            _, filename, ext = get_fn_ext(filepath=path)
            if ext in Options.ALL_VIDEO_FORMAT_OPTIONS.value:
                _ = get_video_meta_data(video_path=path)
                cap = cv2.VideoCapture(path)
                cap.set(1, 0)
                _, self.image = cap.read()

            elif ext in Options.ALL_IMAGE_FORMAT_OPTIONS.value:
                self.image = cv2.imread(path).astype(np.uint8)

            else:
                raise InvalidFileTypeError(
                    msg=f"Cannot crop a {ext} file.", source=self.__class__.__name__
                )

            title = f"Draw ROI video {filename} - Press ESC when drawn"

        self.img_cpy = self.image.copy()

        if title is None:
            self.title = "Draw ROI - Press ESC when drawn"
        else:
            self.title = title

    def mouse_callback(self, event, x, y, flags, param):
        if event == cv2.EVENT_LBUTTONDOWN:
            self.roi_start = (x, y)
            self.roi_end = (x, y)
            self.selecting_roi = True

        elif event == cv2.EVENT_LBUTTONUP:
            self.roi_end = (x, y)
            self.selecting_roi = False

        elif event == cv2.EVENT_MOUSEMOVE and self.selecting_roi:
            self.roi_end = (x, y)
            self.img_cpy = self.image.copy()
            cv2.rectangle(
                self.img_cpy, self.roi_start, self.roi_end, self.clr, self.thickness
            )

    def run(self):
        cv2.namedWindow(self.title, cv2.WINDOW_NORMAL)
        cv2.setMouseCallback(self.title, self.mouse_callback)

        while True:
            if self.selecting_roi or self.img_cpy is None:
                self.img_cpy = self.image.copy()
                if self.roi_start is not None and self.roi_end is not None:
                    cv2.rectangle(
                        self.img_cpy,
                        self.roi_start,
                        self.roi_end,
                        self.clr,
                        self.thickness,
                    )

            cv2.imshow(self.title, self.img_cpy)
            key = cv2.waitKey(1) & 0xFF

            if (
                key == ord("c")
                and self.roi_start is not None
                and self.roi_end is not None
            ):
                selected_roi = self.image[
                    self.roi_start[1] : self.roi_end[1],
                    self.roi_start[0] : self.roi_end[0],
                ].copy()
                reflected_roi = cv2.flip(selected_roi, 1)
                self.image[
                    self.roi_start[1] : self.roi_end[1],
                    self.roi_start[0] : self.roi_end[0],
                ] = reflected_roi
                self.roi_start = None
                self.roi_end = None

            elif key == 27:
                if self.run_checks():
                    cv2.destroyAllWindows()
                    cv2.waitKey(1)
                    break

    def run_checks(self):
        self.top_left = min(self.roi_start[0], self.roi_end[0]), min(
            self.roi_start[1], self.roi_end[1]
        )
        self.bottom_right = max(self.roi_start[0], self.roi_end[0]), max(
            self.roi_start[1], self.roi_end[1]
        )
        self.width = self.bottom_right[0] - self.top_left[0]
        self.height = self.bottom_right[1] - self.top_left[1]

        if (self.width == 0 and self.height == 0) or (
            self.width + self.height + self.top_left[0] + self.top_left[1] == 0
        ):
            CropWarning(
                msg="CROP WARNING: Cropping height and width are both 0. Please try again.",
                source=self.__class__.__name__,
            )
            return False

        else:
            return True


# img_selector = ROISelector(path='/Users/simon/Desktop/envs/troubleshooting/ARES_data/Termite Test 1/Termite Test 1.mp4')
# img_selector.run()

# img_selector = ROISelector(path='/Users/simon/Desktop/compute_overlap.png', clr=(0, 255, 0), thickness=2)
# img_selector.run()

#
# # Global variables to store the ROI coordinates
# roi_start = None
# roi_end = None
# selecting_roi = False
# image_copy = None
#
# def mouse_callback(event, x, y, flags, param):
#     global roi_start, roi_end, selecting_roi, image_copy
#
#     if event == cv2.EVENT_LBUTTONDOWN:
#         roi_start = (x, y)
#         roi_end = (x, y)
#         selecting_roi = True
#
#     elif event == cv2.EVENT_LBUTTONUP:
#         roi_end = (x, y)
#         selecting_roi = False
#
#     elif event == cv2.EVENT_MOUSEMOVE and selecting_roi:
#         roi_end = (x, y)
#         image_copy = image.copy()
#         cv2.rectangle(image_copy, roi_start, roi_end, (0, 255, 0), 2)
#
# # Load your image
# image_path = '/Users/simon/Desktop/compute_overlap.png'
# image = cv2.imread(image_path)
# image_copy = image.copy()
#
# # Create a window and set the mouse callback
# cv2.namedWindow('Select ROI')
# cv2.setMouseCallback('Select ROI', mouse_callback)
#
# while True:
#     if roi_start is not None and roi_end is not None:
#         # Draw the selected ROI on the image copy
#         image_copy = image.copy()
#         cv2.rectangle(image_copy, roi_start, roi_end, (0, 255, 0), 2)
#
#     cv2.imshow('Select ROI', image_copy)
#
#     key = cv2.waitKey(1) & 0xFF
#
#     if key == ord('c') and roi_start is not None and roi_end is not None:
#         # Copy the selected ROI
#         selected_roi = image[roi_start[1]:roi_end[1], roi_start[0]:roi_end[0]].copy()
#
#         # Reflect the ROI horizontally
#         reflected_roi = cv2.flip(selected_roi, 1)
#
#         # Replace the original ROI with the reflected one
#         image[roi_start[1]:roi_end[1], roi_start[0]:roi_end[0]] = reflected_roi
#
#         # Reset ROI coordinates
#         roi_start = None
#         roi_end = None
#
#     elif key == 27:  # Press Esc to exit
#         break
#
# cv2.destroyAllWindows()



__author__ = "Simon Nilsson"

import os
from tkinter import *

from simba.mixins.config_reader import ConfigReader
from simba.mixins.pop_up_mixin import PopUpMixin
from simba.ui.tkinter_functions import (CreateLabelFrameWithIcon, DropDownMenu,
                                        Entry_Box)
from simba.utils.checks import check_float
from simba.utils.enums import ConfigKey, Formats, Keys, Links
from simba.utils.printing import stdout_success


class OutlierSettingsPopUp(PopUpMixin, ConfigReader):
    def __init__(self, config_path: str):
        PopUpMixin.__init__(self, title="OUTLIER SETTINGS")
        ConfigReader.__init__(self, config_path=config_path)
        self.animal_bps = {}
        for animal_name, animal_data in self.animal_bp_dict.items():
            self.animal_bps[animal_name] = [x[:-2] for x in animal_data["X_bps"]]

        self.location_correction_frm = CreateLabelFrameWithIcon(
            parent=self.main_frm,
            header="LOCATION CORRECTION",
            icon_name=Keys.DOCUMENTATION.value,
            icon_link=Links.OULIERS.value,
        )

        bp_entry_cnt, self.criterion_dropdowns = 0, {}
        for animal_cnt, animal_name in enumerate(self.animal_bp_dict.keys()):
            self.criterion_dropdowns[animal_name] = {}
            self.criterion_dropdowns[animal_name]["location_bp_1"] = DropDownMenu(
                self.location_correction_frm,
                "Choose {} body part 1:".format(animal_name),
                self.animal_bps[animal_name],
                "30",
            )
            self.criterion_dropdowns[animal_name]["location_bp_2"] = DropDownMenu(
                self.location_correction_frm,
                "Choose {} body part 2:".format(animal_name),
                self.animal_bps[animal_name],
                "30",
            )
            self.criterion_dropdowns[animal_name]["location_bp_1"].setChoices(
                self.animal_bps[animal_name][0]
            )
            self.criterion_dropdowns[animal_name]["location_bp_2"].setChoices(
                self.animal_bps[animal_name][1]
            )
            self.criterion_dropdowns[animal_name]["location_bp_1"].grid(
                row=bp_entry_cnt, column=0, sticky=NW
            )
            bp_entry_cnt += 1
            self.criterion_dropdowns[animal_name]["location_bp_2"].grid(
                row=bp_entry_cnt, column=0, sticky=NW
            )
            bp_entry_cnt += 1
        self.location_criterion = Entry_Box(
            self.location_correction_frm, "Location criterion: ", "15"
        )
        self.location_criterion.grid(row=bp_entry_cnt, column=0, sticky=NW)
        self.location_correction_frm.grid(row=0, column=0, sticky=NW)

        self.movement_correction_frm = LabelFrame(
            self.main_frm,
            text="MOVEMENT CORRECTION",
            font=("Times", 12, "bold"),
            pady=5,
            padx=5,
        )
        bp_entry_cnt = 0
        for animal_cnt, animal_name in enumerate(self.animal_bp_dict.keys()):
            self.criterion_dropdowns[animal_name]["movement_bp_1"] = DropDownMenu(
                self.movement_correction_frm,
                "Choose {} body part 1:".format(animal_name),
                self.animal_bps[animal_name],
                "30",
            )
            self.criterion_dropdowns[animal_name]["movement_bp_2"] = DropDownMenu(
                self.movement_correction_frm,
                "Choose {} body part 2:".format(animal_name),
                self.animal_bps[animal_name],
                "30",
            )
            self.criterion_dropdowns[animal_name]["movement_bp_1"].setChoices(
                self.animal_bps[animal_name][0]
            )
            self.criterion_dropdowns[animal_name]["movement_bp_2"].setChoices(
                self.animal_bps[animal_name][1]
            )
            self.criterion_dropdowns[animal_name]["movement_bp_1"].grid(
                row=bp_entry_cnt, column=0, sticky=NW
            )
            bp_entry_cnt += 1
            self.criterion_dropdowns[animal_name]["movement_bp_2"].grid(
                row=bp_entry_cnt, column=0, sticky=NW
            )
            bp_entry_cnt += 1
        self.movement_criterion = Entry_Box(
            self.movement_correction_frm, "Location criterion: ", "15"
        )
        self.movement_criterion.grid(row=bp_entry_cnt, column=0, sticky=NW)
        self.movement_correction_frm.grid(row=1, column=0, sticky=NW)

        agg_type_frm = LabelFrame(
            self.main_frm,
            text="AGGREGATION METHOD",
            font=Formats.LABELFRAME_HEADER_FORMAT.value,
            pady=5,
            padx=5,
        )
        self.agg_type_dropdown = DropDownMenu(
            agg_type_frm, "Aggregation method:", ["mean", "median"], "15"
        )
        self.agg_type_dropdown.setChoices("median")
        self.agg_type_dropdown.grid(row=0, column=0, sticky=NW)
        agg_type_frm.grid(row=2, column=0, sticky=NW)

        run_btn = Button(
            self.main_frm,
            text="CONFIRM",
            font=Formats.LABELFRAME_HEADER_FORMAT.value,
            fg="red",
            command=lambda: self.run(),
        )
        run_btn.grid(row=3, column=0, sticky=NW)

        # self.main_frm.mainloop()

    def run(self):
        if self.config.has_section(ConfigKey.OUTLIER_SETTINGS.value):
            self.config.remove_section(ConfigKey.OUTLIER_SETTINGS.value)
        self.config.add_section(ConfigKey.OUTLIER_SETTINGS.value)
        check_float(
            name="LOCATION CRITERION",
            value=self.location_criterion.entry_get,
            min_value=0.0,
        )
        check_float(
            name="MOVEMENT CRITERION",
            value=self.movement_criterion.entry_get,
            min_value=0.0,
        )
        if not self.config.has_section("Outlier settings"):
            self.config.add_section("Outlier settings")
        self.config.set(
            ConfigKey.OUTLIER_SETTINGS.value,
            ConfigKey.MOVEMENT_CRITERION.value,
            str(self.movement_criterion.entry_get),
        )
        self.config.set(
            ConfigKey.OUTLIER_SETTINGS.value,
            ConfigKey.LOCATION_CRITERION.value,
            str(self.location_criterion.entry_get),
        )
        self.config.set(
            ConfigKey.OUTLIER_SETTINGS.value,
            "mean_or_median",
            str(self.agg_type_dropdown.getChoices()),
        )
        for animal_cnt, animal_name in enumerate(self.animal_bp_dict.keys()):
            self.config.set(
                ConfigKey.OUTLIER_SETTINGS.value,
                "movement_bodyPart1_{}".format(animal_name),
                self.criterion_dropdowns[animal_name]["movement_bp_1"].getChoices(),
            )
            self.config.set(
                ConfigKey.OUTLIER_SETTINGS.value,
                "movement_bodyPart2_{}".format(animal_name),
                self.criterion_dropdowns[animal_name]["movement_bp_2"].getChoices(),
            )
            self.config.set(
                ConfigKey.OUTLIER_SETTINGS.value,
                "location_bodyPart1_{}".format(animal_name),
                self.criterion_dropdowns[animal_name]["location_bp_1"].getChoices(),
            )
            self.config.set(
                ConfigKey.OUTLIER_SETTINGS.value,
                "location_bodyPart2_{}".format(animal_name),
                self.criterion_dropdowns[animal_name]["location_bp_2"].getChoices(),
            )
        with open(self.config_path, "w") as f:
            self.config.write(f)

        stdout_success(
            msg="Outlier correction settings updated in the project_config.ini",
            source=self.__class__.__name__,
        )
        self.root.destroy()


# _ = OutlierSettingsPopUp(config_path='/Users/simon/Desktop/envs/troubleshooting/two_black_animals_14bp/project_folder/project_config.ini')



__author__ = "Simon Nilsson"

import ast
import os
from typing import Union

import pandas as pd
from sklearn.ensemble import RandomForestClassifier
from sklearn.model_selection import train_test_split

from simba.mixins.config_reader import ConfigReader
from simba.mixins.train_model_mixin import TrainModelMixin
from simba.utils.checks import check_if_filepath_list_is_empty, check_int
from simba.utils.enums import (ConfigKey, Dtypes, Formats, Methods,
                               MLParamKeys, Options, TagNames)
from simba.utils.printing import SimbaTimer, log_event, stdout_success
from simba.utils.read_write import read_config_entry, write_df


class TrainRandomForestClassifier(ConfigReader, TrainModelMixin):
    """
    Train a single random forest model using hyperparameter setting and evaluation methods
    stored within the SimBA project config .ini file (``global environment``).

    :param str config_path: path to SimBA project config file in Configparser format

    .. note::
       `Tutorial <https://github.com/sgoldenlab/simba/blob/master/docs/Scenario1.md#step-7-train-machine-model>`_

    :example:
    >>> model_trainer = TrainRandomForestClassifier(config_path='MyConfigPath')
    >>> model_trainer.perform_sampling()
    >>> model_trainer.train_model()
    >>> model_trainer.save_model()
    """

    def __init__(self, config_path: Union[str, os.PathLike]):

        ConfigReader.__init__(self, config_path=config_path)
        TrainModelMixin.__init__(self)
        log_event(
            logger_name=str(self.__class__.__name__),
            log_type=TagNames.CLASS_INIT.value,
            msg=self.create_log_msg_from_init_args(locals=locals()),
        )
        self.read_model_settings_from_config(config=self.config)

        check_if_filepath_list_is_empty(
            filepaths=self.target_file_paths,
            error_msg="Zero annotation files found in project_folder/csv/targets_inserted directory, cannot create model.",
        )
        print(
            "Reading in {} annotated files...".format(str(len(self.target_file_paths)))
        )
        self.data_df, self.frm_idx = self.read_all_files_in_folder_mp_futures(
            self.target_file_paths, self.file_type, [self.clf_name]
        )
        self.frm_idx = pd.DataFrame(
            {"VIDEO": list(self.data_df.index), "FRAME_IDX": self.frm_idx}
        )
        self.data_df = self.check_raw_dataset_integrity(
            df=self.data_df, logs_path=self.logs_path
        )
        self.data_df_wo_cords = self.drop_bp_cords(df=self.data_df)
        annotation_cols_to_remove = self.read_in_all_model_names_to_remove(
            self.config, self.clf_cnt, self.clf_name
        )
        self.x_y_df = self.delete_other_annotation_columns(
            self.data_df_wo_cords, list(annotation_cols_to_remove)
        )
        self.class_names = ["Not_" + self.clf_name, self.clf_name]
        self.x_df, self.y_df = self.split_df_to_x_y(self.x_y_df, self.clf_name)
        self.feature_names = self.x_df.columns
        self.check_sampled_dataset_integrity(x_df=self.x_df, y_df=self.y_df)
        print("Number of features in dataset: " + str(len(self.x_df.columns)))
        print(
            "Number of {} frames in dataset: {} ({}%)".format(
                self.clf_name,
                str(self.y_df.sum()),
                str(round(self.y_df.sum() / len(self.y_df), 4) * 100),
            )
        )
        print("Training and evaluating model...")
        self.timer = SimbaTimer(start=True)

    def perform_sampling(self):
        """
        Method for sampling data for training and testing, and perform over and under-sampling of the training sets
        as indicated within the SimBA project config.
        """

        if self.split_type == Methods.SPLIT_TYPE_FRAMES.value:
            self.x_train, self.x_test, self.y_train, self.y_test = train_test_split(
                self.x_df, self.y_df, test_size=self.tt_size
            )
        elif self.split_type == Methods.SPLIT_TYPE_BOUTS.value:
            self.x_train, self.x_test, self.y_train, self.y_test = (
                self.bout_train_test_splitter(
                    x_df=self.x_df, y_df=self.y_df, test_size=self.tt_size
                )
            )
        if self.under_sample_setting == Methods.RANDOM_UNDERSAMPLE.value.lower():
            self.x_train, self.y_train = self.random_undersampler(
                self.x_train, self.y_train, float(self.under_sample_ratio)
            )
        if self.over_sample_setting == Methods.SMOTEENN.value.lower():
            self.x_train, self.y_train = self.smoteen_oversampler(
                self.x_train, self.y_train, float(self.over_sample_ratio)
            )
        elif self.over_sample_setting == Methods.SMOTE.value.lower():
            self.x_train, self.y_train = self.smote_oversampler(
                self.x_train, self.y_train, float(self.over_sample_ratio)
            )

        if self.save_train_test_frm_info:
            train_data = self.frm_idx[
                self.frm_idx.index.isin(self.x_train.index)
            ].set_index("VIDEO")
            test_data = self.frm_idx[
                self.frm_idx.index.isin(self.x_test.index)
            ].set_index("VIDEO")
            write_df(
                df=train_data,
                file_type=Formats.CSV.value,
                save_path=os.path.join(self.eval_out_path, "train_idx.csv"),
            )
            write_df(
                df=test_data,
                file_type=Formats.CSV.value,
                save_path=os.path.join(self.eval_out_path, "test_idx.csv"),
            )
            print(
                f"Frame index for the train and test sets saved in {self.eval_out_path} directory..."
            )

    def train_model(self):
        """
        Method for training single random forest model.
        """

        if self.algo == "RF":
            n_estimators = read_config_entry(
                self.config,
                ConfigKey.CREATE_ENSEMBLE_SETTINGS.value,
                MLParamKeys.RF_ESTIMATORS.value,
                data_type=Dtypes.INT.value,
            )
            max_features = read_config_entry(
                self.config,
                ConfigKey.CREATE_ENSEMBLE_SETTINGS.value,
                MLParamKeys.RF_MAX_FEATURES.value,
                data_type=Dtypes.STR.value,
            )
            if max_features == "None":
                max_features = None
            criterion = read_config_entry(
                self.config,
                ConfigKey.CREATE_ENSEMBLE_SETTINGS.value,
                MLParamKeys.RF_CRITERION.value,
                data_type=Dtypes.STR.value,
                options=Options.CLF_CRITERION.value,
            )
            min_sample_leaf = read_config_entry(
                self.config,
                ConfigKey.CREATE_ENSEMBLE_SETTINGS.value,
                MLParamKeys.MIN_LEAF.value,
                data_type=Dtypes.INT.value,
            )
            compute_permutation_importance = read_config_entry(
                self.config,
                ConfigKey.CREATE_ENSEMBLE_SETTINGS.value,
                MLParamKeys.PERMUTATION_IMPORTANCE.value,
                data_type=Dtypes.STR.value,
                default_value=False,
            )
            generate_learning_curve = read_config_entry(
                self.config,
                ConfigKey.CREATE_ENSEMBLE_SETTINGS.value,
                MLParamKeys.LEARNING_CURVE.value,
                data_type=Dtypes.STR.value,
                default_value=False,
            )
            generate_precision_recall_curve = read_config_entry(
                self.config,
                ConfigKey.CREATE_ENSEMBLE_SETTINGS.value,
                MLParamKeys.PRECISION_RECALL.value,
                data_type=Dtypes.STR.value,
                default_value=False,
            )
            generate_example_decision_tree = read_config_entry(
                self.config,
                ConfigKey.CREATE_ENSEMBLE_SETTINGS.value,
                MLParamKeys.EX_DECISION_TREE.value,
                data_type=Dtypes.STR.value,
                default_value=False,
            )
            generate_classification_report = read_config_entry(
                self.config,
                ConfigKey.CREATE_ENSEMBLE_SETTINGS.value,
                MLParamKeys.CLF_REPORT.value,
                data_type=Dtypes.STR.value,
                default_value=False,
            )
            generate_features_importance_log = read_config_entry(
                self.config,
                ConfigKey.CREATE_ENSEMBLE_SETTINGS.value,
                MLParamKeys.IMPORTANCE_LOG.value,
                data_type=Dtypes.STR.value,
                default_value=False,
            )
            generate_features_importance_bar_graph = read_config_entry(
                self.config,
                ConfigKey.CREATE_ENSEMBLE_SETTINGS.value,
                MLParamKeys.IMPORTANCE_LOG.value,
                data_type=Dtypes.STR.value,
                default_value=False,
            )
            generate_example_decision_tree_fancy = read_config_entry(
                self.config,
                ConfigKey.CREATE_ENSEMBLE_SETTINGS.value,
                MLParamKeys.EX_DECISION_TREE_FANCY.value,
                data_type=Dtypes.STR.value,
                default_value=False,
            )
            generate_shap_scores = read_config_entry(
                self.config,
                ConfigKey.CREATE_ENSEMBLE_SETTINGS.value,
                MLParamKeys.SHAP_SCORES.value,
                data_type=Dtypes.STR.value,
                default_value=False,
            )
            save_meta_data = read_config_entry(
                self.config,
                ConfigKey.CREATE_ENSEMBLE_SETTINGS.value,
                MLParamKeys.RF_METADATA.value,
                data_type=Dtypes.STR.value,
                default_value=False,
            )
            compute_partial_dependency = read_config_entry(
                self.config,
                ConfigKey.CREATE_ENSEMBLE_SETTINGS.value,
                MLParamKeys.PARTIAL_DEPENDENCY.value,
                data_type=Dtypes.STR.value,
                default_value=False,
            )

            if self.config.has_option(
                ConfigKey.CREATE_ENSEMBLE_SETTINGS.value,
                MLParamKeys.CLASS_WEIGHTS.value,
            ):
                class_weights = read_config_entry(
                    self.config,
                    ConfigKey.CREATE_ENSEMBLE_SETTINGS.value,
                    MLParamKeys.CLASS_WEIGHTS.value,
                    data_type=Dtypes.STR.value,
                    default_value=Dtypes.NONE.value,
                )
                if class_weights == "custom":
                    class_weights = ast.literal_eval(
                        read_config_entry(
                            self.config,
                            ConfigKey.CREATE_ENSEMBLE_SETTINGS.value,
                            MLParamKeys.CLASS_CUSTOM_WEIGHTS.value,
                            data_type=Dtypes.STR.value,
                        )
                    )
                    for k, v in class_weights.items():
                        class_weights[k] = int(v)
                if class_weights == Dtypes.NONE.value:
                    class_weights = None
            else:
                class_weights = None

            if generate_learning_curve in Options.PERFORM_FLAGS.value:
                shuffle_splits = read_config_entry(
                    self.config,
                    ConfigKey.CREATE_ENSEMBLE_SETTINGS.value,
                    MLParamKeys.LEARNING_CURVE_K_SPLITS.value,
                    data_type=Dtypes.INT.value,
                    default_value=Dtypes.NAN.value,
                )
                dataset_splits = read_config_entry(
                    self.config,
                    ConfigKey.CREATE_ENSEMBLE_SETTINGS.value,
                    MLParamKeys.LEARNING_DATA_SPLITS.value,
                    data_type=Dtypes.INT.value,
                    default_value=Dtypes.NAN.value,
                )
                check_int(
                    name=MLParamKeys.LEARNING_CURVE_K_SPLITS.value, value=shuffle_splits
                )
                check_int(
                    name=MLParamKeys.LEARNING_DATA_SPLITS.value, value=dataset_splits
                )
            else:
                shuffle_splits, dataset_splits = Dtypes.NAN.value, Dtypes.NAN.value
            if generate_features_importance_bar_graph in Options.PERFORM_FLAGS.value:
                feature_importance_bars = read_config_entry(
                    self.config,
                    ConfigKey.CREATE_ENSEMBLE_SETTINGS.value,
                    MLParamKeys.IMPORTANCE_BARS_N.value,
                    Dtypes.INT.value,
                    Dtypes.NAN.value,
                )
                check_int(
                    name=MLParamKeys.IMPORTANCE_BARS_N.value,
                    value=feature_importance_bars,
                    min_value=1,
                )
            else:
                feature_importance_bars = Dtypes.NAN.value
            (
                shap_target_present_cnt,
                shap_target_absent_cnt,
                shap_save_n,
                shap_multiprocess,
            ) = (None, None, None, None)
            if generate_shap_scores in Options.PERFORM_FLAGS.value:
                shap_target_present_cnt = read_config_entry(
                    self.config,
                    ConfigKey.CREATE_ENSEMBLE_SETTINGS.value,
                    MLParamKeys.SHAP_PRESENT.value,
                    data_type=Dtypes.INT.value,
                    default_value=0,
                )
                shap_target_absent_cnt = read_config_entry(
                    self.config,
                    ConfigKey.CREATE_ENSEMBLE_SETTINGS.value,
                    MLParamKeys.SHAP_ABSENT.value,
                    data_type=Dtypes.INT.value,
                    default_value=0,
                )
                shap_save_n = read_config_entry(
                    self.config,
                    ConfigKey.CREATE_ENSEMBLE_SETTINGS.value,
                    MLParamKeys.SHAP_SAVE_ITERATION.value,
                    data_type=Dtypes.STR.value,
                    default_value=Dtypes.NONE.value,
                )
                shap_multiprocess = read_config_entry(
                    self.config,
                    ConfigKey.CREATE_ENSEMBLE_SETTINGS.value,
                    MLParamKeys.SHAP_MULTIPROCESS.value,
                    data_type=Dtypes.STR.value,
                    default_value="False",
                )
                try:
                    shap_save_n = int(shap_save_n)
                except ValueError:
                    shap_save_n = shap_target_present_cnt + shap_target_absent_cnt
                check_int(
                    name=MLParamKeys.SHAP_PRESENT.value, value=shap_target_present_cnt
                )
                check_int(
                    name=MLParamKeys.SHAP_ABSENT.value, value=shap_target_absent_cnt
                )

            self.rf_clf = RandomForestClassifier(
                n_estimators=n_estimators,
                max_depth=self.rf_max_depth,
                max_features=max_features,
                n_jobs=-1,
                criterion=criterion,
                min_samples_leaf=min_sample_leaf,
                bootstrap=True,
                verbose=1,
                class_weight=class_weights,
            )

            print(f"Fitting {self.clf_name} model...")
            self.rf_clf = self.clf_fit(
                clf=self.rf_clf, x_df=self.x_train, y_df=self.y_train
            )

            if compute_permutation_importance in Options.PERFORM_FLAGS.value:
                self.calc_permutation_importance(
                    self.x_test,
                    self.y_test,
                    self.rf_clf,
                    self.feature_names,
                    self.clf_name,
                    self.eval_out_path,
                )
            if generate_learning_curve in Options.PERFORM_FLAGS.value:
                self.calc_learning_curve(
                    x_y_df=self.x_y_df,
                    clf_name=self.clf_name,
                    shuffle_splits=shuffle_splits,
                    dataset_splits=dataset_splits,
                    tt_size=self.tt_size,
                    rf_clf=self.rf_clf,
                    save_dir=self.eval_out_path,
                )

            if generate_precision_recall_curve in Options.PERFORM_FLAGS.value:
                self.calc_pr_curve(
                    self.rf_clf,
                    self.x_test,
                    self.y_test,
                    self.clf_name,
                    self.eval_out_path,
                )
            if generate_example_decision_tree in Options.PERFORM_FLAGS.value:
                self.create_example_dt(
                    self.rf_clf,
                    self.clf_name,
                    self.feature_names,
                    self.class_names,
                    self.eval_out_path,
                )
            if generate_classification_report in Options.PERFORM_FLAGS.value:
                self.create_clf_report(
                    self.rf_clf,
                    self.x_test,
                    self.y_test,
                    self.class_names,
                    self.eval_out_path,
                )
            if generate_features_importance_log in Options.PERFORM_FLAGS.value:
                self.create_x_importance_log(
                    self.rf_clf, self.feature_names, self.clf_name, self.eval_out_path
                )
            if generate_features_importance_bar_graph in Options.PERFORM_FLAGS.value:
                self.create_x_importance_bar_chart(
                    self.rf_clf,
                    self.feature_names,
                    self.clf_name,
                    self.eval_out_path,
                    feature_importance_bars,
                )
            if generate_example_decision_tree_fancy in Options.PERFORM_FLAGS.value:
                self.dviz_classification_visualization(
                    self.x_train,
                    self.y_train,
                    self.clf_name,
                    self.class_names,
                    self.eval_out_path,
                )
            if generate_shap_scores in Options.PERFORM_FLAGS.value:
                if not shap_multiprocess in Options.PERFORM_FLAGS.value:
                    self.create_shap_log(
                        ini_file_path=self.config_path,
                        rf_clf=self.rf_clf,
                        x_df=self.x_train,
                        y_df=self.y_train,
                        x_names=self.feature_names,
                        clf_name=self.clf_name,
                        cnt_present=shap_target_present_cnt,
                        cnt_absent=shap_target_absent_cnt,
                        save_it=shap_save_n,
                        save_path=self.eval_out_path,
                    )
                else:
                    self.create_shap_log_mp(
                        ini_file_path=self.config_path,
                        rf_clf=self.rf_clf,
                        x_df=self.x_train,
                        y_df=self.y_train,
                        x_names=self.feature_names,
                        clf_name=self.clf_name,
                        cnt_present=shap_target_present_cnt,
                        cnt_absent=shap_target_absent_cnt,
                        save_path=self.eval_out_path,
                    )

            if compute_partial_dependency in Options.PERFORM_FLAGS.value:
                self.partial_dependence_calculator(
                    clf=self.rf_clf,
                    x_df=self.x_train,
                    clf_name=self.clf_name,
                    save_dir=self.eval_out_path,
                )

            if save_meta_data in Options.PERFORM_FLAGS.value:
                meta_data_lst = [
                    self.clf_name,
                    criterion,
                    max_features,
                    min_sample_leaf,
                    n_estimators,
                    compute_permutation_importance,
                    generate_classification_report,
                    generate_example_decision_tree,
                    generate_features_importance_bar_graph,
                    generate_features_importance_log,
                    generate_precision_recall_curve,
                    save_meta_data,
                    generate_learning_curve,
                    dataset_splits,
                    shuffle_splits,
                    feature_importance_bars,
                    self.over_sample_ratio,
                    self.over_sample_setting,
                    self.tt_size,
                    self.split_type,
                    self.under_sample_ratio,
                    self.under_sample_setting,
                    str(class_weights),
                    self.rf_max_depth,
                ]

                self.create_meta_data_csv_training_one_model(
                    meta_data_lst, self.clf_name, self.eval_out_path
                )

    def save_model(self) -> None:
        """
        Method for saving pickled RF model. The model is saved in the `models/generated_models` directory
        of the SimBA project tree.
        """

        self.timer.stop_timer()
        self.save_rf_model(self.rf_clf, self.clf_name, self.model_dir_out)
        stdout_success(
            msg=f"Classifier {self.clf_name} saved in models/generated_models directory",
            elapsed_time=self.timer.elapsed_time_str,
            source=self.__class__.__name__,
        )
        stdout_success(
            msg=f"Evaluation files are in models/generated_models/model_evaluations folders",
            source=self.__class__.__name__,
        )


# test = TrainRandomForestClassifier(config_path='/Users/simon/Desktop/envs/troubleshooting/two_black_animals_14bp/project_folder/project_config.ini')
# test.perform_sampling()
# test.train_model()
# test.save_model()


# test = TrainRandomForestClassifier(config_path='/Users/simon/Desktop/envs/troubleshooting/two_black_animals_14bp/project_folder/project_config.ini')
# test.perform_sampling()
# test.train_model()
# test.save_model()

# test = TrainSingleModel(config_path='/Users/simon/Desktop/envs/troubleshooting/locomotion/project_folder/project_config.ini')
# test.perform_sampling()
# test.train_model()
# test.save_model()


# test = TrainSingleModel(config_path='/Users/simon/Desktop/envs/troubleshooting/prueba/project_folder/project_config.ini')
# test.perform_sampling()
# test.train_model()
# test.save_model()

# test = TrainSingleModel(config_path='/Users/simon/Desktop/envs/troubleshooting/naresh/project_folder/project_config.ini')
# test.perform_sampling()
# test.train_model()
# test.save_model()

# test = TrainSingleModel(config_path='/Users/simon/Desktop/envs/troubleshooting/Lucas/project_folder/project_config.ini')
# test.perform_sampling()
# test.train_model()
# test.save`_model()



__author__ = "Simon Nilsson"

import os
import threading
from copy import copy

import matplotlib.pyplot as plt

from simba.mixins.config_reader import ConfigReader
from simba.plotting.tools.tkinter_tools import InteractiveVideoPlotterWindow
from simba.utils.checks import check_file_exist_and_readable
from simba.utils.enums import Paths
from simba.utils.errors import ColumnNotFoundError, InvalidInputError
from simba.utils.read_write import get_fn_ext, read_df


class InteractiveProbabilityGrapher(ConfigReader):
    """
    Launch interactive GUI for classifier probability inspection.

    :param str config_path: path to SimBA project config file in Configparser format
    :param str file_path: Data with classification probability field.
    :param str model_path: Path to classifier used to create probability field.

    .. note::
       `Validation tutorial <https://github.com/sgoldenlab/simba/blob/master/docs/Scenario1.md#critical-validation-step-before-running-machine-model-on-new-data>`__.
        .. image:: _static/img/interactive_probability_plot.png
           :width: 450
           :align: center


    Examples
    ----------
    >>> interactive_plotter = InteractiveProbabilityGrapher(config_path=r'MyConfigPath', file_path='MyFeatureFilepath', model_path='MyPickledClassifier.sav')
    >>> interactive_plotter.run()
    """

    def __init__(self, config_path: str, file_path: str, model_path: str):
        super().__init__(config_path=config_path)
        check_file_exist_and_readable(file_path=file_path)
        check_file_exist_and_readable(file_path=model_path)
        self.file_path, self.model_path = file_path, model_path
        self.click_counter = 0
        _, self.clf_name, _ = get_fn_ext(filepath=self.model_path)
        if self.clf_name not in self.clf_names:
            raise InvalidInputError(
                msg=f"The classifier {self.clf_name} is not a classifier in the SimBA project: {self.clf_names}"
            )
        self.data_path = os.path.join(
            self.project_path,
            Paths.CLF_DATA_VALIDATION_DIR.value,
            os.path.basename(self.file_path),
        )
        check_file_exist_and_readable(self.data_path)
        _, video_name, _ = get_fn_ext(filepath=file_path)
        self.data_df = read_df(self.data_path, self.file_type)
        if f"Probability_{self.clf_name}" not in self.data_df.columns:
            raise ColumnNotFoundError(
                column_name=f"Probability_{self.clf_name}", file_name=self.data_path
            )
        self.p_arr = self.data_df[["Probability_{}".format(self.clf_name)]].to_numpy()
        current_video_file_path = self.find_video_of_file(
            video_dir=self.video_dir, filename=video_name
        )
        self.video_frm = InteractiveVideoPlotterWindow(
            video_path=current_video_file_path, p_arr=self.p_arr
        )

    @staticmethod
    def __click_event(event):
        global current_x_cord
        if (event.dblclick) and (event.button == 1) and (type(event.xdata) != None):
            current_x_cord = int(event.xdata)

    def run(self):
        import matplotlib

        matplotlib.use("TkAgg")
        global current_x_cord
        probability_txt = (
            f"Selected frame: {str(0)}, {self.clf_name} probability: {self.p_arr[0][0]}"
        )
        plt_title = f"Click on the points of the graph to display the corresponding video frame. \n {probability_txt}"
        current_x_cord, prior_x_cord = None, None

        fig, ax = plt.subplots()
        ax.plot(self.p_arr)
        plt.xlabel("frame #", fontsize=16)
        plt.ylabel(f"{self.clf_name} probability", fontsize=16)
        plt.title(plt_title)
        plt.grid()
        line = None
        fig.canvas.draw()
        fig.canvas.flush_events()

        while True:
            _ = fig.canvas.mpl_connect(
                "button_press_event", lambda event: self.__click_event(event)
            )
            if current_x_cord != prior_x_cord:
                prior_x_cord = copy(current_x_cord)
                probability_txt = "Selected frame: {}, {} probability: {}".format(
                    str(current_x_cord),
                    self.clf_name,
                    str(self.p_arr[current_x_cord][0]),
                )
                plt_title = f"Click on the points of the graph to display the corresponding video frame. \n {probability_txt}"
                self.video_frm.load_new_frame(frm_cnt=int(current_x_cord))
                if line != None:
                    line.remove()
                plt.title(plt_title)
                line = plt.axvline(x=current_x_cord, color="r")
            fig.canvas.draw()
            fig.canvas.flush_events()
            plt.ion()
            threading.Thread(plt.show()).start()
            plt.pause(0.0001)


# test = InteractiveProbabilityGrapher(config_path=r'/Users/simon/Desktop/envs/troubleshooting/naresh/project_folder/project_config.ini',
#                                      file_path='/Users/simon/Desktop/envs/troubleshooting/naresh/project_folder/csv/features_extracted/SF2.csv',
#                                      model_path='/Users/simon/Desktop/envs/troubleshooting/naresh/models/generated_models/Top.sav')
# test.create_plots()



__author__ = "Simon Nilsson"

import glob
import os
from pathlib import Path
from typing import Dict, Optional, Tuple

import cv2
import pandas as pd

from simba.utils.checks import check_if_filepath_list_is_empty
from simba.utils.data import create_color_palette
from simba.utils.enums import Formats
from simba.utils.errors import InvalidInputError
from simba.utils.lookups import get_color_dict
from simba.utils.printing import SimbaTimer, stdout_success
from simba.utils.read_write import (find_video_of_file, get_fn_ext,
                                    get_video_meta_data, read_df)

ACCEPTED_DIRECTORIES = [
    "input_csv",
    "outlier_corrected_movement",
    "outlier_corrected_movement_location",
]


def create_video_from_dir(
    in_directory: str,
    out_directory: str,
    circle_size: int,
    clr_attr: Optional[Dict[str, Tuple[int, int, int]]] = None,
):
    """
    Create pose-estimation visualizations from data within a SimBA project folder.

    :param str in_directory: Path to SimBA project directory containing pose-estimation data in parquet or CSV format.
    :param str out_directory: Directory to where to save the pose-estimation videos.
    :param int Size of the circles denoting the location of the pose-estimated body-parts.
    :param Optional[dict] clr_attr: Python dict where animals are keys and color attributes values. E.g., {'Animal_1':  (255, 107, 198)}. If None,
                                    random palettes will be used.

    :example:
    >>> create_video_from_dir(in_directory='InputDirectory', out_directory='OutputDirectory', circle_size=5, clr_attr=None)

    """

    if (os.path.basename(in_directory)) not in ACCEPTED_DIRECTORIES:
        raise InvalidInputError(
            msg=f"{in_directory} is not an accepted directory for visualization. OPTIONS: {ACCEPTED_DIRECTORIES}"
        )

    files_found = glob.glob(in_directory + "/*." + "csv") + glob.glob(
        in_directory + "/*." + "parquet"
    )
    check_if_filepath_list_is_empty(
        filepaths=files_found,
        error_msg="SIMBA ERROR: 0 files found in {} in CSV or PARQUET file format".format(
            in_directory
        ),
    )
    print("Processing {} videos ...".format(str(len(files_found))))
    timer = SimbaTimer()
    timer.start_timer()
    for video_cnt, file_path in enumerate(files_found):
        dir_name, file_name, ext = get_fn_ext(file_path)
        video_dir = os.path.join(Path(file_path).parents[2], "videos")
        video_file_path = find_video_of_file(video_dir, file_name)
        pose_df = read_df(file_path, ext[1:].lower())
        if os.path.basename(in_directory) == "input_csv":
            pose_df.columns = list(pose_df.loc["coords"])

        pose_df = pose_df.apply(pd.to_numeric, errors="coerce")
        pose_df = pose_df.fillna(0).reset_index(drop=True)
        bp_lst_of_lst = [
            list(pose_df.columns)[i : i + 3] for i in range(0, len(pose_df.columns), 3)
        ]
        color_list = create_color_palette(
            increments=len(bp_lst_of_lst), pallete_name="Set1"
        )
        animal_bp_dict = {}
        if clr_attr:
            clrs = get_color_dict()
            animal_bp_dict = {}
            for animal in range(1, len(clr_attr.keys()) + 1):
                animal_bp_dict["Animal_{}".format(str(animal))] = {}
                animal_bp_dict["Animal_{}".format(str(animal))]["bps"] = []
                animal_bp_dict["Animal_{}".format(str(animal))]["color"] = clrs[
                    clr_attr["Animal_{}".format(str(animal))]
                ]
                for bp in bp_lst_of_lst:
                    if str(animal) in bp[0].split("_")[-2]:
                        animal_bp_dict["Animal_{}".format(str(animal))]["bps"].append(
                            bp
                        )
        video_meta_data = get_video_meta_data(video_path=video_file_path)
        cap = cv2.VideoCapture(video_file_path)
        if not os.path.exists(out_directory):
            os.makedirs(out_directory)

        save_path = os.path.join(out_directory, file_name + ".mp4")
        fourcc = cv2.VideoWriter_fourcc(*Formats.MP4_CODEC.value)
        writer = cv2.VideoWriter(
            save_path,
            fourcc,
            video_meta_data["fps"],
            (video_meta_data["width"], video_meta_data["height"]),
        )

        frm_cnt = 0
        while cap.isOpened():
            ret, frame = cap.read()
            if ret:
                if not clr_attr:
                    for cnt, bp in enumerate(bp_lst_of_lst):
                        bp_tuple = (
                            int(pose_df.at[frm_cnt, bp[0]]),
                            int(pose_df.at[frm_cnt, bp[1]]),
                        )
                        cv2.circle(frame, bp_tuple, circle_size, color_list[cnt], -1)
                else:
                    for animal_name, animal_data in animal_bp_dict.items():
                        for bp in animal_data["bps"]:
                            bp_tuple = (
                                int(pose_df.at[frm_cnt, bp[0]]),
                                int(pose_df.at[frm_cnt, bp[1]]),
                            )
                            cv2.circle(
                                frame, bp_tuple, circle_size, animal_data["color"], -1
                            )
                frm_cnt += 1
                writer.write(frame)
                print(
                    "Video: {} / {} Frame: {} / {}".format(
                        str(video_cnt + 1),
                        str(len(files_found)),
                        str(frm_cnt),
                        str(len(pose_df)),
                    )
                )
            else:
                break
        print("{} complete...".format(file_name))
        cap.release()
        writer.release()
    timer.stop_timer()
    stdout_success(
        msg=f"All pose videos complete. Results located in {str(out_directory)} directory",
        elapsed_time=timer.elapsed_time_str,
    )


# create_video_from_dir(in_directory=r'/Users/simon/Desktop/envs/troubleshooting/naresh/project_folder/csv/outlier_corrected_movement_location',
#                       out_directory=r'/Users/simon/Desktop/envs/troubleshooting/naresh/project_folder/csv/features_extracted/test',
#                       circle_size=5,
#                       clr_attr=None)
